package datavisualizer;

import javax.swing.JTable;
import javax.swing.JPanel;
import java.awt.Dimension;
import java.awt.Color;
import javawrapper.ModelException;
import java.util.Vector;

/**
 * Produces a table of storm results. A storm table requires the detailed output
 * file to have package data saved from the "Storm Results" grid. The table has
 * four columns: timestep, num storms, avg severity of most severe storm, avg
 * severity of least severe storm.
 * 
 * This does all timesteps at once.
 * 
 * Copyright: Copyright (c) Charles D. Canham 2007 Company: Institute of
 * Ecosystem Studies
 * 
 * @author Lora E. Murphy
 * @version 1.0
 * 
 * <br>
 * Edit history: <br>
 * ------------------ <br>
 * June 4, 2007: Created (LEM)
 */

public class StormDataRequest extends DataRequest {
	
	/** The data that will be appearing in the table - this allows us to have
	 * a copy saved for file saving etc */
	private String[][] mp_sTable;

	/** Place for storing storm data as it comes in, number of X cells by number
	 * of Y cells */
	private Vector[][] mp_oData;

  private String[] mp_sColHeaders = {"Timestep", "Num storms", 
  		"Mean of most severe", "Mean of least severe"}; 

	/** Name of the grid from which the data will be collected. */
	private String m_sThisGrid;
	
	/**
	 * The total number of timesteps. We keep track of this so we know if
	 * something has changed (as in real-time data visualization) so we can update
	 * appropriately.
	 */
	private int m_iNumTimesteps = -1;

	/** Code for "1dmg_index" package float data member */
	private int m_i1DmgIndexCode;

	/**
	 * Constructor
	 * 
	 * @param sChartName
	 *          Name of the chart
	 * @param oManager
	 *          Detailed output file manager
	 * @throws ModelException
	 *           not really, but I have to declare this.
	 */
	public StormDataRequest(String sChartName, DetailedOutputFileManager oManager)
			throws ModelException {
		super(sChartName, oManager);

		m_sThisGrid = "Storm Damage";

		// Set that this chart doesn't write only one timestep at once
		m_bShowOneTimestep = false;

		// Get the number of timesteps
		m_iNumTimesteps = oManager.GetNumberOfActualTimesteps();

		ClearData();
	}

	/**
	 * Writes the table's data to tab-delimited text.
	 * 
	 * @param jOut
	 *          java.io.FileWriter The file to write to.
	 * @throws java.io.IOException
	 *           if there's a problem writing the file.
	 */
	protected void WriteChartDataToFile(java.io.FileWriter jOut)
			throws java.io.IOException {

		int iNumRows, iRow, iCol;
		
		jOut.write(mp_sColHeaders[0]);
		for (iCol = 1; iCol < mp_sColHeaders.length; iCol++)
			jOut.write("\t" + mp_sColHeaders[iCol]);
		jOut.write("\n");

		// Write table
		iNumRows = mp_sTable.length;
		for (iRow = 0; iRow < iNumRows; iRow++) {
			jOut.write(mp_sTable[iRow][0]);
			for (iCol = 1; iCol < mp_sTable[iRow].length; iCol++) {
				jOut.write("\t" + mp_sTable[iRow][iCol]);
			}
			jOut.write("\n");
		}
	}

	/**
	 * This accepts a package float value. It figures out what it is, then adds it
	 * to the appropriate location.
	 * 
	 * @param sGridName
	 *          Name of the grid for this data
	 * @param iX
	 *          X number of the cell from which this value came
	 * @param iY
	 *          Y number of the cell from which this value came
	 * @param iCode
	 *          Data member code of this value.
	 * @param fVal
	 *          Value.
	 */
	public void AddGridPackageFloatData(String sGridName, int iX, int iY,
			int iCode, float fVal) {
		if (sGridName.equals(m_sThisGrid)) {
			// Is this storm severity?
			if (m_i1DmgIndexCode == iCode) {
				mp_oData[iX][iY].add(new Float(fVal));
			}
		}
	}

	/**
	 * This wants package floats.
	 * 
	 * @return True.
	 */
	public boolean WantAnyGridPackageFloats() {
		return true;
	}

	/**
	 * Accepts a grid package float data member code for future reference when
	 * passed float data members.
	 * 
	 * @param sGridName
	 *          Name of the grid
	 * @param sLabel
	 *          The label of the data member.
	 * @param iCode
	 *          The data member code.
	 */
	public void AddGridPackageFloatDataMemberCode(String sGridName,
			String sLabel, int iCode) {
		if (sGridName.equals(m_sThisGrid)) {
			// Is this storm severity?
			if (sLabel.equals("1dmg_index")) {
				m_i1DmgIndexCode = iCode;
			}
		}
	}

	/**
	 * Creates a table of the results that have been collected.
	 * 
	 * @param oLegend
	 *          The species legend.
	 * @param sChartTitle
	 *          The chart title.
	 * @return A JInternalFrame with the table present in it.
	 * @throws datavisualizer.ModelException
	 * 
	 * <br>
	 * Edit history: <br>
	 * ------------------ <br>
	 * March 20, 2006: Created (LEM)
	 */
	ModelInternalFrame DrawChart(Legend oLegend, String sChartTitle)
			throws ModelException {

		m_oChartFrame = new ModelInternalFrame(sChartTitle, oLegend);
		m_oChartFrame.setVisible(true); //so this won't get deleted
		m_oChartFrame.setContentPane(CreateTable((DetailedOutputLegend)oLegend));
		m_oChartFrame = DataGrapher.AddFileMenu(m_oChartFrame, this,
				m_bShowOneTimestep);
		return m_oChartFrame;
	}

	/**
	 * Does nothing unless the number of timesteps is different, in which case the
	 * table is recreated. This chart does not respond to regular chart update
	 * events, since it already shows data for all timesteps.
	 * 
	 * @param oLegend
	 *          Legend The legend for this chart.
	 * @throws ModelException
	 *           Won't be thrown.
	 */
	void UpdateChart(Legend oLegend) throws javawrapper.ModelException {
		if (((DetailedOutputLegend) oLegend).GetNumberOfTimesteps() != m_iNumTimesteps) {
			m_iNumTimesteps = ((DetailedOutputLegend) oLegend).GetNumberOfTimesteps();
			// Update the chart
			m_oChartFrame.setContentPane(CreateTable((DetailedOutputLegend)oLegend));
		}
	}

	/**
	 * Turns the accumulated data into a table for viewing.
	 * 
	 * @return JPanel The panel containing the table.
	 */
	protected JPanel CreateTable(DetailedOutputLegend oLegend) {
		JPanel jPanel = null;
		try {
			m_oChartFrame.setCursor(java.awt.Cursor
					.getPredefinedCursor(java.awt.Cursor.WAIT_CURSOR));
			
			java.text.NumberFormat jFormat = java.text.NumberFormat.getInstance();
      jFormat.setMaximumFractionDigits(3);

			DetailedOutputFileManager oManager = oLegend
					.GetDetailedOutputFileManager();
			float fLengthXCells = oManager.GetGridXCellLength(m_sThisGrid),
			      fLengthYCells = oManager.GetGridYCellLength(m_sThisGrid),
			      fLengthXPlot = oManager.GetXPlotLength(),
			      fLengthYPlot = oManager.GetYPlotLength(),
			      fAvg, fAvgMaxSeverity, fAvgMinSeverity;
			int iNumTimesteps = oLegend.GetNumberOfTimesteps(), 
			    iTimestepToReturnTo = oLegend.GetCurrentTimestep(),
			    iNumXCells = (int)java.lang.Math.ceil(fLengthXPlot / fLengthXCells),
			    iNumYCells = (int)java.lang.Math.ceil(fLengthYPlot / fLengthYCells),
			    iNumStorms,
			    iX, iY, i,
			    iNumCols = 4, iTs;

			mp_sTable = new String[iNumTimesteps+1][iNumCols];
			mp_oData = new Vector[iNumXCells][iNumYCells];
			for (iX = 0; iX < iNumXCells; iX++)
				for (iY = 0; iY < iNumYCells; iY++)
					mp_oData[iX][iY] = new Vector(0);

			// Force the processing of each timestep
			for (iTs = 0; iTs <= iNumTimesteps; iTs++) {

				for (iX = 0; iX < iNumXCells; iX++)
					for (iY = 0; iY < iNumYCells; iY++)
						mp_oData[iX][iY].clear();

				//Make the file manager parse this file
				oManager.ReadFile(iTs);
				
        //Get the number of storms that occurred from the first cell - 
				//they should all be the same
				iNumStorms = mp_oData[0][0].size();
				
				if (iNumStorms == 0) {
					fAvgMaxSeverity = 0;
					fAvgMinSeverity = 0;
				} else {
					fAvgMaxSeverity = -1;
					fAvgMinSeverity = -1;
					
				  //Get the average severity of the storm values we've collected
					fAvg = 0;
				  for (i = 0; i < iNumStorms; i++) {
				  	for (iX = 0; iX < iNumXCells; iX++) {
							for (iY = 0; iY < iNumYCells; iY++) {
								fAvg += ((Float) mp_oData[iX][iY].elementAt(i)).floatValue();
							}	
				  	}	   
					  fAvg /= (iNumXCells * iNumYCells);

				    //If the avg is less than the current minimum, put this as
					  //the minimum
				    if (fAvgMinSeverity < 0 || fAvgMinSeverity > fAvg)
					    fAvgMinSeverity = fAvg;

				    //If the avg is greater than the current maximum, put this as 
				    //the maximum
				    if (fAvgMaxSeverity < fAvg)
					    fAvgMaxSeverity = fAvg;
				  }
				}

				mp_sTable[iTs][0] = String.valueOf(iTs);
				mp_sTable[iTs][1] = String.valueOf(mp_oData[0][0].size());
				mp_sTable[iTs][2] = jFormat.format(fAvgMaxSeverity);
				mp_sTable[iTs][3] = jFormat.format(fAvgMinSeverity);

			}

			// Refresh all the existing charts back to the way they were
			oManager.ReadFile(iTimestepToReturnTo);
			oManager.UpdateCharts();

			JTable jTable = new JTable(mp_sTable, mp_sColHeaders);
			jTable.setName("ResultsTable");
			jTable.setGridColor(Color.WHITE);
			jTable.setBackground(Color.WHITE);
			jTable.setFont(new javawrapper.ModelFont());
			jTable.getTableHeader().setFont(new javawrapper.ModelFont());
			jTable.setPreferredSize(new Dimension(500, (int) jTable
					.getPreferredSize().height));

			// Create the panel and add the table to it.
			jPanel = new JPanel(new java.awt.BorderLayout());
			javax.swing.JScrollPane jScroller = new javax.swing.JScrollPane(jTable);
			jPanel.add(jScroller, java.awt.BorderLayout.CENTER);
			jPanel.setBackground(Color.WHITE);
			
		} catch (javawrapper.ModelException oErr) {
			;
		} // fail silently
		m_oChartFrame.setCursor(java.awt.Cursor.getPredefinedCursor(java.awt.Cursor.
        DEFAULT_CURSOR));
		return jPanel;
	}

	/**
   * Does nothing, because this table doesn't operate on the detailed output
   * file timescale.
   * @throws ModelException Won't throw it.
   */
  void ClearData() throws javawrapper.ModelException {
    ;
  }
}
