package javawrapper;

import java.util.Vector;

/**
 * Represents a single behavior in the core.  There is a one-to-one
 * relationship between behaviors in the core model and objects of this class.
 * Unlike in the core model, however, this class is not meant to be extended
 * and tailored.
 * <p>
 * Objects of this class act as placeholders for behaviors, keeping track of
 * whether or not they are enabled and to whom they apply.  They are "dumb",
 * not performing any additional functions.  They rely on their
 * BehaviorTypeBase owners to keep track of their data.
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 * <br>April 30, 2004: Made version into a float (duh) (LEM)
 * <br>July 20, 2004:  Added new tree data members (LEM)
 */

public class Behavior {

  /**A set of SpeciesTypeCombo objects to which this behavior is applied -
   * or none if it is not applied to trees*/
  protected Vector mp_oTreesAppliesTo = new Vector(0);
  /**A set of strings with the name of Grids to which this behavior is
   * applied - or none if it is not applied to grids*/
  protected Vector mp_oGridsAppliesTo = new Vector(0);
  /**The data objects which are required when this behavior is enabled
   * (i.e. the individual pieces of data that must have values)*/
  protected Vector mp_oRequiredData = new Vector(0);
  /**The new tree data members that this behavior adds, above and beyond the
   * basic tree population data members.  This is a vector of
   * DataMember objects.*/
  protected Vector mp_oNewTreeDataMembers = new Vector(0);

  /**If true, and a tree has no species/type combos, it automatically becomes
   * disabled.  Otherwise, it has to do with grids and enablement must be
   * specifically set.*/
  public boolean m_bMustHaveTrees = true;
  /**
   * Whether or not the current parameter file uses this behavior - and thus
   * whether it will get called during the setup sequence
   */
  public boolean m_bIsEnabled = false;
  /**Whether or not this behavior can be edited automatically (true) or
   * must be edited with its own dialog (false)*/
  public boolean m_bCanEditAutomatically = true;

  /**This is what will be displayed as the name of a given behavior in the
   * GUI.  Shouldn't be too long - max 5 words or so*/
  protected String m_sDescriptor;
  /**Key string - short unique identifier*/
  protected String m_sKey;
  /**String which is used to identify this behavior in the parameter file*/
  protected String m_sParFileTag;

  /**For each tree type, whether or not this behavior can be applied to
   * it.  This defaults to true for seedlings, saplings, adults, and snags,
   * and false for all others.*/
  protected boolean[] mp_bCanApplyToTreeType;

  /**Version of this behavior*/
  protected float m_fVersion;

  /**Minimum version of this behavior*/
  protected float m_fMinVersion = (float) 1.0;

  /**
   * Constructor.
   * @param sKey Key string - short unique identifier.
   * @param sDescriptor Long description that will show up in GUI displays.
   * @param sParFileTag Parameter file XML tag (no brackets).
   * @param fVersion C++ object version number.
   * @param fMinVersion C++ object minimum version number
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>April 30, 2004: Updated version to be a float (LEM)
   */
  public Behavior(String sKey, String sDescriptor, String sParFileTag,
                  float fVersion, float fMinVersion) {

    m_sKey = sKey;
    m_sDescriptor = sDescriptor;
    m_sParFileTag = sParFileTag;
    m_fVersion = fVersion;
    m_fMinVersion = fMinVersion;

    //Create the array which says which tree types this can apply to
    mp_bCanApplyToTreeType = new boolean[TreePopulation.GetNumberOfTypes()];
    int i;
    for (i = 0; i < mp_bCanApplyToTreeType.length; i++) {
      mp_bCanApplyToTreeType[i] = false;
    }

    //Now default to true for seedlings, saplings, and adults
    mp_bCanApplyToTreeType[TreePopulation.SEEDLING] = true;
    mp_bCanApplyToTreeType[TreePopulation.SAPLING] = true;
    mp_bCanApplyToTreeType[TreePopulation.ADULT] = true;
    mp_bCanApplyToTreeType[TreePopulation.SNAG] = true;
  }

  /**
   * Constructor.
   * @param sKey Key string - short unique identifier.
   * @param sDescriptor Long description that will show up in GUI displays.
   * @param sParFileTag Parameter file XML tag (no brackets).
   * @param fVersion C++ object version number.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>April 30, 2004: Updated version to be a float (LEM)
   */
  public Behavior(String sKey, String sDescriptor, String sParFileTag,
                  float fVersion) {

    this(sKey, sDescriptor, sParFileTag, fVersion, (float) 1.0);
  }

  /**
   * Behavior performs an internal check to make sure its settings are
   * consistent and logical.  If m_bMustHaveTrees is set to true,
   * it is enabled, and it has no trees, it throws an error.
   * @throws ModelException if the above condition is true.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void Validate() throws ModelException {
    if (m_bIsEnabled &&
        m_bMustHaveTrees &&
        mp_oTreesAppliesTo.size() == 0) {
      throw( new ModelException(ErrorGUI.DATA_MISSING, "JAVA",
                                "The behavior \"" + m_sDescriptor +
                                "\" must either be applied to trees or removed"
                                + " from the run."));
    }
  }

  /**
   * Sets whether this behavior can apply to a given tree type.
   * @param iType Tree type.
   * @param bCanApply True if this can be applied to a tree type; false if not.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetCanApplyTo(int iType, boolean bCanApply) {
    mp_bCanApplyToTreeType[iType] = bCanApply;
  }

  /**
   * Gets the key string.
   * @return Key string.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetKey() {
    return m_sKey;
  }

  /**
   * Gets the parameter file name for this behavior.
   * @return Parameter file name for this behavior.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetParameterFileBehaviorName() {
    return m_sParFileTag;
  }

  /**
   * Gets behavior's parameter file version.
   * @return Parameter file version.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>April 30, 2004: Made version into a float (LEM)
   */
  public float GetVersion() {
    return m_fVersion;
  }

  /**
   * Gets behavior's parameter file minimum version.
   * @return Parameter file minimum version.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 30, 2004: Created (LEM)
   */
  public float GetMinimumVersion() {
    return m_fMinVersion;
  }

  /**
   * Gets the descriptor string.
   * @return The descriptor string.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetDescriptor() {
    return m_sDescriptor;
  }

  /**
   * Sets the descriptor string.
   * @param s New descriptor string.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetDescriptor(String s) {
    m_sDescriptor = s;
  }

  /**
   * Gets the number of species/type combos to which this behavior applies.
   * @return the number of species/type combos.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetNumberOfCombos() {
    if (null == mp_oTreesAppliesTo) {
      return 0;
    }
    else {
      return mp_oTreesAppliesTo.size();
    }
  }

  /**
   * Gets the number of grids to which this behavior applies.
   * @return the number of grids.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetNumberOfGrids() {
    if (null == mp_oGridsAppliesTo) {
      return 0;
    }
    else {
      return mp_oGridsAppliesTo.size();
    }
  }

  /**
   * Deletes the SpeciesTypeCombo at the given index.  If this removes the last
   * combo, and the behavior has its m_bMustHaveTrees flag set to true, this
   * sets m_bIsEnabled to false.
   * @param iIndex The index number of the SpeciesTypeCombo.
   * @throws ModelException if the index number is invalid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void DeleteSpeciesTypeCombo(int iIndex) throws ModelException {
    if (iIndex < 0 || iIndex >= mp_oTreesAppliesTo.size()) {
      throw (new ModelException(ErrorGUI.BAD_DATA, "JAVA",
         "Invalid index passed to DeleteSpeciesTypeCombo."));
    }
    mp_oTreesAppliesTo.remove(iIndex);

    if (mp_oTreesAppliesTo.size() == 0 && m_bMustHaveTrees) {
      m_bIsEnabled = false;
    }
  }

  /**
   * Empties the species/type combo list.  If this behavior has its
   * m_bMustHaveTrees flag set to true, this sets m_bIsEnabled to false.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void ClearSpeciesTypeCombos() {
    if (null != mp_oTreesAppliesTo) {
      mp_oTreesAppliesTo.clear();
    }

    if (m_bMustHaveTrees) {
      m_bIsEnabled = false;
    }
  }

  /**
   * Returns the SpeciesTypeCombo at the given index of the species/type
   * combo list.
   * @param iIndex Index of combo desired.
   * @return The SpeciesTypeCombo object requested.
   * @throws ModelException if the index is not valid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public SpeciesTypeCombo GetSpeciesTypeCombo(int iIndex) throws ModelException {
    if (iIndex < 0 || iIndex >= mp_oTreesAppliesTo.size()) {
      throw( new ModelException(ErrorGUI.BAD_DATA, "JAVA",
      "Invalid index passed to GetSpeciesTypeCombo."));
    }
    SpeciesTypeCombo oCombo = (SpeciesTypeCombo) mp_oTreesAppliesTo.elementAt(
        iIndex);
    return oCombo;
  }

  /**
   * Returns the Grid at the given index of the grid applies to list.
   * @param iIndex Index of grid desired.
   * @return The Grid object requested.
   * @throws ModelException if the index is not valid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Grid GetGrid(int iIndex) throws ModelException {
    if (iIndex < 0 || iIndex >= mp_oGridsAppliesTo.size()) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                               "Invalid index passed to GetGrid."));
    }
    Grid oGrid = (Grid) mp_oGridsAppliesTo.elementAt(iIndex);
    return oGrid;
  }

  /**
   * Returns the required data object at a given index.
   * @param iIndex Index.
   * @return Required data.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Object GetRequiredData(int iIndex) {
    return mp_oRequiredData.elementAt(iIndex);
  }

  /**
   * Gets the number of required data objects for this behavior.
   * @return Number of required data objects.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public int GetNumberOfRequiredDataObjects() {
    return mp_oRequiredData.size();
  }

  /**
   * Adds a new species/type combo to the list to which this behavior applies.
   * If this particular species/type combo is already on the list, it's not
   * added again.  Calling this causes this behavior to become enabled.
   * @param oCombo The new species/type combo.
   * @throws ModelException if the tree type in the combo cannot be applied to
   * this behavior according to mp_bCanApplyToTreeType.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddSpeciesTypeCombo(SpeciesTypeCombo oCombo) throws
      ModelException {

    //Check to see if this type is allowed
    if (mp_bCanApplyToTreeType[oCombo.GetType()] == false) {
      throw( new ModelException(ErrorGUI.BAD_ARGUMENT, "JAVA",
                               "The behavior \"" + m_sDescriptor +
                               "\" cannot be applied to trees of type \"" +
                     TreePopulation.GetTypeNameFromCode(oCombo.GetType()) +
                               "\"."));
    }

    //Check to see if this combo has already been added
    int i;
    for (i = 0; i < mp_oTreesAppliesTo.size(); i++) {
      if (oCombo.equals(mp_oTreesAppliesTo.elementAt(i))) {
        return;
      }
    }

    mp_oTreesAppliesTo.add(oCombo);

    m_bIsEnabled = true;
  }

  /**
   * Adds a new Grid to the list to which this behavior applies.  If this
   * particular grid is already on the list, it's replaced. Calling
   * this method does NOT automatically cause the behavior to become enabled,
   * because grids are more of an inherent property of behaviors.  For this
   * reason, a Behavior should always be notified of its Grids whether or
   * not it is enabled.
   * @param oGrid Grid to add.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>July 21, 2004:  Made it so a duplicate grid replaces an existing of
   * the same name.  This allows for grids to be updated. (LEM)
   */
  public void AddGrid(Grid oGrid) {

    //Check to see if this grid has already been added
    Grid oExistingGrid;
    int i;
    for (i = 0; i < mp_oGridsAppliesTo.size(); i++) {
      oExistingGrid = (Grid) mp_oGridsAppliesTo.elementAt(i);
      if (oGrid.GetName().equals(oExistingGrid.GetName())) {
        mp_oGridsAppliesTo.remove(i);
      }
    }

    mp_oGridsAppliesTo.add(oGrid);
  }

  /**
   * Adds a piece of required data to the list.
   * @param oData Data to add.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddRequiredData(Object oData) {
    //Check to see if this data has already been added
    /*    int i;
        for (i = 0; i < mp_oRequiredData.size(); i++) {
          if (mp_oRequiredData.elementAt(i).equals(oData)) {
            return;
          }
        }
     */
    mp_oRequiredData.add(oData);
  }

  /**
   * Returns which species to which this behavior is applied.  If multiple tree
   * types are applied, a species is considered applied to if any, not all, of
   * the types goes with that species.
   * @param oPop Tree population.
   * @return An array, sized total number of species, with a boolean for each
   * species number as to whether or not this behavior applies to it.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public boolean[] GetWhichSpeciesUsed(TreePopulation oPop) {
    SpeciesTypeCombo oCombo;
    boolean[] p_bFlags = new boolean[oPop.GetNumberOfSpecies()];
    int i;

    //Initialize all to false
    for (i = 0; i < p_bFlags.length; i++) {
      p_bFlags[i] = false;

    }
    for (i = 0; i < mp_oTreesAppliesTo.size(); i++) {
      oCombo = (SpeciesTypeCombo) mp_oTreesAppliesTo.elementAt(i);
      p_bFlags[oCombo.GetSpecies()] = true;
    }

    return p_bFlags;
  }

  /**
   * Override to make a clone of this object.
   * @return Deep clone of this object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Object clone() {
    Behavior oClone = new Behavior(m_sKey, m_sDescriptor, m_sParFileTag,
                                   m_fVersion);
    SpeciesTypeCombo oCombo;
    DataMember oDataMember;
    int i;
    oClone.m_bIsEnabled = m_bIsEnabled;
    oClone.m_bMustHaveTrees = m_bMustHaveTrees;
    oClone.m_bCanEditAutomatically = m_bCanEditAutomatically;

    //Copy species/type combos
    for (i = 0; i < mp_oTreesAppliesTo.size(); i++) {
      oCombo = (SpeciesTypeCombo) mp_oTreesAppliesTo.elementAt(i);
      oClone.mp_oTreesAppliesTo.add( (SpeciesTypeCombo) oCombo.clone());
    }

    //Copy grids - don't need to clone these individually
    for (i = 0; i < mp_oGridsAppliesTo.size(); i++) {
      oClone.AddGrid( (Grid) mp_oGridsAppliesTo.elementAt(i));
    }

    //Copy required data - don't need to clone these individually
    for (i = 0; i < mp_oRequiredData.size(); i++) {
      oClone.AddRequiredData(mp_oRequiredData.elementAt(i));
    }

    //Declare and copy the "applies to" array
    oClone.mp_bCanApplyToTreeType = new boolean[mp_bCanApplyToTreeType.length];
    for (i = 0; i < mp_bCanApplyToTreeType.length; i++) {
      oClone.mp_bCanApplyToTreeType[i] = mp_bCanApplyToTreeType[i];
    }

    //Copy the data members
    for (i = 0; i < mp_oNewTreeDataMembers.size(); i++) {
      oDataMember = (DataMember) mp_oNewTreeDataMembers.elementAt(i);
      oClone.mp_oNewTreeDataMembers.add(oDataMember.clone());
    }

    return oClone;
  }
}
