package javawrapper;

import java.util.Vector;
import java.io.BufferedWriter;

/**
 * This class represents grids in the core model.  Objects of this class
 * control data saving for grids.
 * <p>
 * This will keep track of a custom cell length in the X and Y directions.  It
 * is possible to change this at any time, but results could be disastrous if
 * there is existing grid map data; it will not be updated to reflect the
 * change.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */

public class Grid {

  protected DataMember[]
      /**
       * This is a list of the data members in a grid
       * and thus available for saving in detailed output files etc.
       */
      mp_oDataMembers,
      /**
       * This is a list of the package data members in a grid
       * and thus available for saving in detailed output files etc.
       */
      mp_oPackageDataMembers;

  protected String[]
      /**Code names of int data members*/
      mp_sIntDataMembers,
      /**Code names of float data members*/
      mp_sFloatDataMembers,
      /**Code names of char data members*/
      mp_sCharDataMembers,
      /**Code names of bool data members*/
      mp_sBoolDataMembers,
      /**Code names of package int data members, if different from main list*/
      mp_sPackageIntDataMembers,
      /**Code names of package float data members, if different from main list*/
      mp_sPackageFloatDataMembers,
      /**Code names of package char data members, if different from main list*/
      mp_sPackageCharDataMembers,
      /**Code names of package bool data members, if different from main list*/
      mp_sPackageBoolDataMembers;

  /**This is the grid's name - it should match the one in the code*/
  protected String m_sGridName;

  protected float
      /**Length of cells in the X direction - optional*/
      m_fLengthXCells,
      /**Length of cells in the Y direction - optional*/
      m_fLengthYCells;

  protected boolean
      /**Whether or not this grid has been edited*/
      m_bEdited = false;

  protected Vector
      /**Holds GridValue objects to be written to a grid map*/
      mp_oGridVals,
      /**Index position for integer data members.  Vector position equals
       * the data member position in the map being read, and vector value is
       * the index for GridValue.*/
      mp_iGridIntTransforms,
      /**Index position for float data members.  Vector position equals
       * the data member position in the map being read, and vector value is
       * the index for GridValue.*/
      mp_iGridFloatTransforms,
      /**Index position for char data members.  Vector position equals
       * the data member position in the map being read, and vector value is
       * the index for GridValue.*/
      mp_iGridCharTransforms,
      /**Index position for bool data members.  Vector position equals
       * the data member position in the map being read, and vector value is
       * the index for GridValue.*/
      mp_iGridBoolTransforms,
      /**Index position for package integer data members.  Vector position
       * equals the data member position in the map being read, and vector
       * value is the index for GridValue.*/
      mp_iGridPackageIntTransforms,
      /**Index position for package float data members.  Vector position equals
       * the data member position in the map being read, and vector value is
       * the index for GridValue.*/
      mp_iGridPackageFloatTransforms,
      /**Index position for package char data members.  Vector position equals
       * the data member position in the map being read, and vector value is
       * the index for GridValue.*/
      mp_iGridPackageCharTransforms,
      /**Index position for package bool data members.  Vector position equals
       * the data member position in the map being read, and vector value is
       * the index for GridValue.*/
      mp_iGridPackageBoolTransforms;

  /**
   * Constructor.
   * @param p_oMembers The data members for this grid.
   * @param p_oPackageMembers The package data members for this grid, or NULL
   * if there are no package data members.
   * @param sGridName The name of the grid.  This should match the name of the
   * grid in the code.
   * @param fXCellLength Length of cells in the X direction, in m
   * @param fYCellLength Length of cells in the Y direction, in m
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Grid(String sGridName, DataMember[] p_oMembers,
              DataMember[] p_oPackageMembers, float fXCellLength,
              float fYCellLength) {
    mp_oDataMembers = p_oMembers;
    mp_oPackageDataMembers = p_oPackageMembers;
    m_sGridName = sGridName;

    mp_oGridVals = new Vector(0);

    m_fLengthXCells = fXCellLength;
    m_fLengthYCells = fYCellLength;

    //Break up the data members into type
    int iNumFloats = 0, iNumInts = 0, iNumChars = 0, iNumBools = 0, i;
    if (mp_oDataMembers != null) {
      for (i = 0; i < mp_oDataMembers.length; i++) {
        if (mp_oDataMembers[i].GetType() == DataMember.BOOLEAN) {
          iNumBools++;
        }
        else if (mp_oDataMembers[i].GetType() == DataMember.INTEGER) {
          iNumInts++;
        }
        else if (mp_oDataMembers[i].GetType() == DataMember.FLOAT) {
          iNumFloats++;
        }
        else if (mp_oDataMembers[i].GetType() == DataMember.CHAR) {
          iNumChars++;
        }
      }

      if (iNumBools > 0) {
        mp_sBoolDataMembers = new String[iNumBools];
        mp_iGridBoolTransforms = new Vector(iNumBools);
      }
      if (iNumFloats > 0) {
        mp_sFloatDataMembers = new String[iNumFloats];
        mp_iGridFloatTransforms = new Vector(iNumFloats);
      }
      if (iNumChars > 0) {
        mp_sCharDataMembers = new String[iNumChars];
        mp_iGridCharTransforms = new Vector(iNumChars);
      }
      if (iNumInts > 0) {
        mp_sIntDataMembers = new String[iNumInts];
        mp_iGridIntTransforms = new Vector(iNumInts);
      }

      iNumFloats = 0;
      iNumInts = 0;
      iNumChars = 0;
      iNumBools = 0;
      for (i = 0; i < mp_oDataMembers.length; i++) {
        if (mp_oDataMembers[i].GetType() == DataMember.BOOLEAN) {
          mp_sBoolDataMembers[iNumBools] = mp_oDataMembers[i].
              GetCodeName();
          mp_iGridBoolTransforms.add(new Integer(iNumBools));
          iNumBools++;
        }
        else if (mp_oDataMembers[i].GetType() == DataMember.INTEGER) {
          mp_sIntDataMembers[iNumInts] = mp_oDataMembers[i].GetCodeName();
          mp_iGridIntTransforms.add(new Integer(iNumInts));
          iNumInts++;
        }
        else if (mp_oDataMembers[i].GetType() == DataMember.FLOAT) {
          mp_sFloatDataMembers[iNumFloats] = mp_oDataMembers[i].
              GetCodeName();
          mp_iGridFloatTransforms.add(new Integer(iNumFloats));
          iNumFloats++;
        }
        else if (mp_oDataMembers[i].GetType() == DataMember.CHAR) {
          mp_sCharDataMembers[iNumChars] = mp_oDataMembers[i].
              GetCodeName();
          mp_iGridCharTransforms.add(new Integer(iNumChars));
          iNumChars++;
        }
      }
    }

    //Repeat for packages
    if (mp_oPackageDataMembers != null) {
      //Break up the data members into type
      iNumFloats = 0;
      iNumInts = 0;
      iNumChars = 0;
      iNumBools = 0;
      for (i = 0; i < mp_oPackageDataMembers.length; i++) {
        if (mp_oPackageDataMembers[i].GetType() == DataMember.BOOLEAN) {
          iNumBools++;
        }
        else if (mp_oPackageDataMembers[i].GetType() == DataMember.INTEGER) {
          iNumInts++;
        }
        else if (mp_oPackageDataMembers[i].GetType() == DataMember.FLOAT) {
          iNumFloats++;
        }
        else if (mp_oPackageDataMembers[i].GetType() == DataMember.CHAR) {
          iNumChars++;
        }
      }

      if (iNumBools > 0) {
        mp_sPackageBoolDataMembers = new String[iNumBools];
        mp_iGridPackageBoolTransforms = new Vector(iNumBools);
      }
      if (iNumFloats > 0) {
        mp_sPackageFloatDataMembers = new String[iNumFloats];
        mp_iGridPackageFloatTransforms = new Vector(iNumFloats);
      }
      if (iNumChars > 0) {
        mp_sPackageCharDataMembers = new String[iNumChars];
        mp_iGridPackageCharTransforms = new Vector(iNumChars);
      }
      if (iNumInts > 0) {
        mp_sPackageIntDataMembers = new String[iNumInts];
        mp_iGridPackageIntTransforms = new Vector(iNumInts);
      }

      iNumFloats = 0;
      iNumInts = 0;
      iNumChars = 0;
      iNumBools = 0;
      for (i = 0; i < mp_oPackageDataMembers.length; i++) {
        if (mp_oPackageDataMembers[i].GetType() == DataMember.BOOLEAN) {
          mp_sPackageBoolDataMembers[iNumBools] = mp_oPackageDataMembers[i].
              GetCodeName();
          mp_iGridPackageBoolTransforms.add(new Integer(iNumBools));
          iNumBools++;
        }
        else if (mp_oPackageDataMembers[i].GetType() == DataMember.INTEGER) {
          mp_sPackageIntDataMembers[iNumInts] = mp_oPackageDataMembers[i].
              GetCodeName();
          mp_iGridPackageIntTransforms.add(new Integer(iNumInts));
          iNumInts++;
        }
        else if (mp_oPackageDataMembers[i].GetType() == DataMember.FLOAT) {
          mp_sPackageFloatDataMembers[iNumFloats] = mp_oPackageDataMembers[i].
              GetCodeName();
          mp_iGridPackageFloatTransforms.add(new Integer(iNumFloats));
          iNumFloats++;
        }
        else if (mp_oPackageDataMembers[i].GetType() == DataMember.CHAR) {
          mp_sPackageCharDataMembers[iNumChars] = mp_oPackageDataMembers[i].
              GetCodeName();
          mp_iGridPackageCharTransforms.add(new Integer(iNumChars));
          iNumChars++;
        }
      }
    }

  }

  /**
   * Gets the code for a float data member.
   * @param sCodeName String Code name (NOT display name) of data member.
   * @return int Code, or -1 if the name is not recognized.
   */
  int GetFloatCode(String sCodeName) {
    int i, iReturn = -1;
    if (mp_sFloatDataMembers != null) {
      for (i = 0; i < mp_sFloatDataMembers.length; i++) {
        if (mp_sFloatDataMembers[i].equals(sCodeName)) {
          iReturn = i;
          break;
        }
      }
    }
    return iReturn;
  }

  /**
   * Gets the code for an int data member.
   * @param sCodeName String Code name (NOT display name) of data member.
   * @return int Code, or -1 if the name is not recognized.
   */
  int GetIntCode(String sCodeName) {
    int i, iReturn = -1;
    if (mp_sIntDataMembers != null) {
      for (i = 0; i < mp_sIntDataMembers.length; i++) {
        if (mp_sIntDataMembers[i].equals(sCodeName)) {
          iReturn = i;
          break;
        }
      }
    }
    return iReturn;
  }

  /**
   * Gets the code for a bool data member.
   * @param sCodeName String Code name (NOT display name) of data member.
   * @return int Code, or -1 if the name is not recognized.
   */
  int GetBoolCode(String sCodeName) {
    int i, iReturn = -1;
    if (mp_sBoolDataMembers != null) {
      for (i = 0; i < mp_sBoolDataMembers.length; i++) {
        if (mp_sBoolDataMembers[i].equals(sCodeName)) {
          iReturn = i;
          break;
        }
      }
    }
    return iReturn;
  }

  /**
   * Gets the code for a char data member.
   * @param sCodeName String Code name (NOT display name) of data member.
   * @return int Code, or -1 if the name is not recognized.
   */
  int GetCharCode(String sCodeName) {
    int i, iReturn = -1;
    if (mp_sCharDataMembers != null) {
      for (i = 0; i < mp_sCharDataMembers.length; i++) {
        if (mp_sCharDataMembers[i].equals(sCodeName)) {
          iReturn = i;
          break;
        }
      }
    }
    return iReturn;
  }



  /**
   * Sets the XML grid map code for a float data member.
   * @param sDataMember Data member code name.
   * @param iIndex Code number.
   * @throws ModelException if the data member isn't recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridFloatCode(String sDataMember, int iIndex) throws
      ModelException {
    int i, iOfficialIndex = -1;

    if (mp_sFloatDataMembers == null) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }
    //Find the data member
    for (i = 0; i < mp_sFloatDataMembers.length; i++) {
      if (mp_sFloatDataMembers[i].equals(sDataMember)) {
        iOfficialIndex = i;
        break;
      }
    }
    if (iOfficialIndex == -1) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }

    if (mp_iGridFloatTransforms.size() <= iIndex) {
      mp_iGridFloatTransforms.setSize(iIndex + 1);
    }
    mp_iGridFloatTransforms.remove(iIndex);
    mp_iGridFloatTransforms.add(iIndex, new Integer(iOfficialIndex));
  }

  /**
   * Sets the XML grid map code for an int data member.
   * @param sDataMember Data member code name.
   * @param iIndex Code number.
   * @throws ModelException if the data member isn't recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridIntCode(String sDataMember, int iIndex) throws
      ModelException {
    int i, iOfficialIndex = -1;

    if (mp_sIntDataMembers == null) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }
    //Find the data member
    for (i = 0; i < mp_sIntDataMembers.length; i++) {
      if (mp_sIntDataMembers[i].equals(sDataMember)) {
        iOfficialIndex = i;
        break;
      }
    }
    if (iOfficialIndex == -1) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));

    }

    if (mp_iGridIntTransforms.size() <= iIndex) {
      mp_iGridIntTransforms.setSize(iIndex + 1);
    }
    mp_iGridIntTransforms.remove(iIndex);
    mp_iGridIntTransforms.add(iIndex, new Integer(iOfficialIndex));
  }

  /**
   * Sets the XML grid map code for a char data member.
   * @param sDataMember Data member code name.
   * @param iIndex Code number.
   * @throws ModelException if the data member isn't recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridCharCode(String sDataMember, int iIndex) throws
      ModelException {
    int i, iOfficialIndex = -1;

    if (mp_sCharDataMembers == null) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }
    //Find the data member
    for (i = 0; i < mp_sCharDataMembers.length; i++) {
      if (mp_sCharDataMembers[i].equals(sDataMember)) {
        iOfficialIndex = i;
        break;
      }
    }
    if (iOfficialIndex == -1) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }

    if (mp_iGridCharTransforms.size() <= iIndex) {
      mp_iGridCharTransforms.setSize(iIndex + 1);
    }
    mp_iGridCharTransforms.remove(iIndex);
    mp_iGridCharTransforms.add(iIndex, new Integer(iOfficialIndex));
  }

  /**
   * Sets the XML grid map code for a bool data member.
   * @param sDataMember Data member code name.
   * @param iIndex Code number.
   * @throws ModelException if the data member isn't recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridBoolCode(String sDataMember, int iIndex) throws
      ModelException {
    int i, iOfficialIndex = -1;

    if (mp_sBoolDataMembers == null) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }
    //Find the data member
    for (i = 0; i < mp_sBoolDataMembers.length; i++) {
      if (mp_sBoolDataMembers[i].equals(sDataMember)) {
        iOfficialIndex = i;
        break;
      }
    }
    if (iOfficialIndex == -1) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }

    if (mp_iGridBoolTransforms.size() <= iIndex) {
      mp_iGridBoolTransforms.setSize(iIndex + 1);
    }
    mp_iGridBoolTransforms.remove(iIndex);
    mp_iGridBoolTransforms.add(iIndex, new Integer(iOfficialIndex));
  }

  /**
   * Sets the XML grid map code for a package float data member.
   * @param sDataMember Data member code name.
   * @param iIndex Code number.
   * @throws ModelException if the data member isn't recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridPackageFloatCode(String sDataMember, int iIndex) throws
      ModelException {
    int i, iOfficialIndex = -1;

    if (mp_sPackageFloatDataMembers == null) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }
    //Find the data member
    for (i = 0; i < mp_sPackageFloatDataMembers.length; i++) {
      if (mp_sPackageFloatDataMembers[i].equals(sDataMember)) {
        iOfficialIndex = i;
        break;
      }
    }
    if (iOfficialIndex == -1) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }

    if (mp_iGridPackageFloatTransforms.size() <= iIndex) {
      mp_iGridPackageFloatTransforms.setSize(iIndex + 1);
    }
    mp_iGridPackageFloatTransforms.remove(iIndex);
    mp_iGridPackageFloatTransforms.add(iIndex, new Integer(iOfficialIndex));
  }

  /**
   * Sets the XML grid map code for a package integer data member.
   * @param sDataMember Data member code name.
   * @param iIndex Code number.
   * @throws ModelException if the data member isn't recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridPackageIntCode(String sDataMember, int iIndex) throws
      ModelException {
    int i, iOfficialIndex = -1;

    if (mp_sPackageIntDataMembers == null) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }
    //Find the data member
    for (i = 0; i < mp_sPackageIntDataMembers.length; i++) {
      if (mp_sPackageIntDataMembers[i].equals(sDataMember)) {
        iOfficialIndex = i;
        break;
      }
    }
    if (iOfficialIndex == -1) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }

    if (mp_iGridPackageIntTransforms.size() <= iIndex) {
      mp_iGridPackageIntTransforms.setSize(iIndex + 1);
    }
    mp_iGridPackageIntTransforms.remove(iIndex);
    mp_iGridPackageIntTransforms.add(iIndex, new Integer(iOfficialIndex));
  }

  /**
   * Sets the XML grid map code for a package char data member.
   * @param sDataMember Data member code name.
   * @param iIndex Code number.
   * @throws ModelException if the data member isn't recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridPackageCharCode(String sDataMember, int iIndex) throws
      ModelException {
    int i, iOfficialIndex = -1;

    if (mp_sPackageCharDataMembers == null) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }
    //Find the data member
    for (i = 0; i < mp_sPackageCharDataMembers.length; i++) {
      if (mp_sPackageCharDataMembers[i].equals(sDataMember)) {
        iOfficialIndex = i;
        break;
      }
    }
    if (iOfficialIndex == -1) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }

    if (mp_iGridPackageCharTransforms.size() <= iIndex) {
      mp_iGridPackageCharTransforms.setSize(iIndex + 1);
    }
    mp_iGridPackageCharTransforms.remove(iIndex);
    mp_iGridPackageCharTransforms.add(iIndex, new Integer(iOfficialIndex));
  }

  /**
   * Sets the XML grid map code for a package bool data member.
   * @param sDataMember Data member code name.
   * @param iIndex Code number.
   * @throws ModelException if the data member isn't recognized.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridPackageBoolCode(String sDataMember, int iIndex) throws
      ModelException {
    int i, iOfficialIndex = -1;

    if (mp_sPackageBoolDataMembers == null) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }
    //Find the data member
    for (i = 0; i < mp_sPackageBoolDataMembers.length; i++) {
      if (mp_sPackageBoolDataMembers[i].equals(sDataMember)) {
        iOfficialIndex = i;
        break;
      }
    }
    if (iOfficialIndex == -1) {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA", "For grid " +
                               m_sGridName + ", unrecognized data \"" +
                               sDataMember + "\" in the grid map."));
    }

    if (mp_iGridPackageBoolTransforms.size() <= iIndex) {
      mp_iGridPackageBoolTransforms.setSize(iIndex + 1);
    }
    mp_iGridPackageBoolTransforms.remove(iIndex);
    mp_iGridPackageBoolTransforms.add(iIndex, new Integer(iOfficialIndex));
  }

  /**
   * Sets a grid map integer value.  This will assume that the code needs
   * to go through the transform array - i.e. it needs translating from the
   * local grid map reference point.
   * @param iX X grid cell coordinate.
   * @param iY Y grid cell coordinate.
   * @param iCode Index at which to set the value.
   * @param iValue Value to set.
   * @param oPlot Plot object.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridValue(int iX, int iY, int iCode, Integer iValue,
                           Plot oPlot) throws ModelException {

    m_bEdited = true;

    GridValue oGridVal = GetGridValue(iX, iY, oPlot);

    Integer iNewCode = (Integer) mp_iGridIntTransforms.elementAt(iCode);

    oGridVal.SetValue(iNewCode.intValue(), iValue);
  }

  /**
   * Sets a grid map float value.  This will assume that the code needs
   * to go through the transform array - i.e. it needs translating from the
   * local grid map reference point.
   * @param iX X grid cell coordinate.
   * @param iY Y grid cell coordinate.
   * @param iCode Index at which to set the value.
   * @param fValue Value to set.
   * @param oPlot Plot object.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridValue(int iX, int iY, int iCode, Float fValue, Plot oPlot) throws
      ModelException {

    m_bEdited = true;

    GridValue oGridVal = GetGridValue(iX, iY, oPlot);

    Integer iNewCode = (Integer) mp_iGridFloatTransforms.elementAt(iCode);

    oGridVal.SetValue(iNewCode.intValue(), fValue);
  }

  /**
   * Sets a grid map char value.  This will assume that the code needs
   * to go through the transform array - i.e. it needs translating from the
   * local grid map reference point.
   * @param iX X grid cell coordinate.
   * @param iY Y grid cell coordinate.
   * @param iCode Index at which to set the value.
   * @param sValue Value to set.
   * @param oPlot Plot object.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridValue(int iX, int iY, int iCode, String sValue, Plot oPlot) throws
      ModelException {

    m_bEdited = true;

    GridValue oGridVal = GetGridValue(iX, iY, oPlot);

    Integer iNewCode = (Integer) mp_iGridCharTransforms.elementAt(iCode);

    oGridVal.SetValue(iNewCode.intValue(), sValue);
  }

  /**
   * Sets a grid map bool value.  This will assume that the code needs
   * to go through the transform array - i.e. it needs translating from the
   * local grid map reference point.
   * @param iX X grid cell coordinate.
   * @param iY Y grid cell coordinate.
   * @param iCode Index at which to set the value.
   * @param bValue Value to set.
   * @param oPlot Plot object.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridValue(int iX, int iY, int iCode, Boolean bValue,
                           Plot oPlot) throws ModelException {

    m_bEdited = true;

    GridValue oGridVal = GetGridValue(iX, iY, oPlot);

    Integer iNewCode = (Integer) mp_iGridBoolTransforms.elementAt(iCode);

    oGridVal.SetValue(iNewCode.intValue(), bValue);
  }

  /**
   * Sets a grid map package integer value.  This will assume that the code
   * needs to go through the transform array - i.e. it needs translating from
   * the local grid map reference point.
   * @param iX X grid cell coordinate.
   * @param iY Y grid cell coordinate.
   * @param iPackageIndex Package index number.
   * @param iCode Index at which to set the value.
   * @param iValue Value to set.
   * @param oPlot Plot object.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridPackageValue(int iX, int iY, int iPackageIndex, int iCode,
                                  Integer iValue,
                                  Plot oPlot) throws ModelException {

    m_bEdited = true;

    GridValue oGridVal = GetGridValue(iX, iY, oPlot);

    PackageGridValue oPackageVal = GetPackageGridValue(oGridVal, iPackageIndex);

    Integer iNewCode = (Integer) mp_iGridPackageIntTransforms.elementAt(iCode);

    oPackageVal.SetValue(iNewCode.intValue(), iValue);
  }

  /**
   * Sets a grid map package float value.  This will assume that the code
   * needs to go through the transform array - i.e. it needs translating from
   * the local grid map reference point.
   * @param iX X grid cell coordinate.
   * @param iY Y grid cell coordinate.
   * @param iPackageIndex Package index number.
   * @param iCode Index at which to set the value.
   * @param fValue Value to set.
   * @param oPlot Plot object.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridPackageValue(int iX, int iY, int iPackageIndex, int iCode,
                                  Float fValue, Plot oPlot) throws
      ModelException {

    m_bEdited = true;

    GridValue oGridVal = GetGridValue(iX, iY, oPlot);

    PackageGridValue oPackageVal = GetPackageGridValue(oGridVal, iPackageIndex);

    Integer iNewCode = (Integer) mp_iGridPackageFloatTransforms.elementAt(iCode);

    oPackageVal.SetValue(iNewCode.intValue(), fValue);
  }

  /**
   * Sets a grid map package char value.  This will assume that the code
   * needs to go through the transform array - i.e. it needs translating from
   * the local grid map reference point.
   * @param iX X grid cell coordinate.
   * @param iY Y grid cell coordinate.
   * @param iPackageIndex Package index number.
   * @param iCode Index at which to set the value.
   * @param sValue Value to set.
   * @param oPlot Plot object.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridPackageValue(int iX, int iY, int iPackageIndex, int iCode,
                                  String sValue, Plot oPlot) throws
      ModelException {

    m_bEdited = true;

    GridValue oGridVal = GetGridValue(iX, iY, oPlot);

    PackageGridValue oPackageVal = GetPackageGridValue(oGridVal, iPackageIndex);

    Integer iNewCode = (Integer) mp_iGridPackageCharTransforms.elementAt(iCode);

    oPackageVal.SetValue(iNewCode.intValue(), sValue);
  }

  /**
   * Sets a grid map package bool value.  This will assume that the code
   * needs to go through the transform array - i.e. it needs translating from
   * the local grid map reference point.
   * @param iX X grid cell coordinate.
   * @param iY Y grid cell coordinate.
   * @param iPackageIndex Package index number.
   * @param iCode Index at which to set the value.
   * @param bValue Value to set.
   * @param oPlot Plot object.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetGridPackageValue(int iX, int iY, int iPackageIndex, int iCode,
                                  Boolean bValue, Plot oPlot) throws
      ModelException {

    m_bEdited = true;

    GridValue oGridVal = GetGridValue(iX, iY, oPlot);

    PackageGridValue oPackageVal = GetPackageGridValue(oGridVal, iPackageIndex);

    Integer iNewCode = (Integer) mp_iGridPackageBoolTransforms.elementAt(iCode);

    oPackageVal.SetValue(iNewCode.intValue(), bValue);
  }

  /**
   * Finds a requested package for a grid cell, or creates a new one if it
   * does not already exist.
   * @param oCell Cell for which to get the package.
   * @param iPackageIndex Index of the package to get in the package list of
   * the cell, starting at 0.
   * @return PackageGridValue object.
   * @throws ModelException if the coordinates are invalid, or if the package
   * index is higher than 1 + the number of packages already in the cell.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected PackageGridValue GetPackageGridValue(GridValue oCell,
                                                 int iPackageIndex) throws
      ModelException {

    //Is there a package at that index?
    if (iPackageIndex < oCell.mp_oPackages.size()) {
      return (PackageGridValue) oCell.mp_oPackages.elementAt(iPackageIndex);
    }

    if (iPackageIndex == oCell.mp_oPackages.size()) {
      int iNumBools, iNumFloats, iNumInts, iNumChars;
      if (mp_sPackageFloatDataMembers == null) {
        iNumFloats = 0;
      }
      else {
        iNumFloats = mp_sPackageFloatDataMembers.length;
      }
      if (mp_sPackageIntDataMembers == null) {
        iNumInts = 0;
      }
      else {
        iNumInts = mp_sPackageIntDataMembers.length;
      }
      if (mp_sPackageCharDataMembers == null) {
        iNumChars = 0;
      }
      else {
        iNumChars = mp_sPackageCharDataMembers.length;
      }
      if (mp_sPackageBoolDataMembers == null) {
        iNumBools = 0;
      }
      else {
        iNumBools = mp_sPackageBoolDataMembers.length;
      }

      PackageGridValue oPackage = new PackageGridValue(iNumFloats, iNumInts,
          iNumChars, iNumBools);
      oCell.mp_oPackages.add(oPackage);
      return oPackage;
    }
    else {
      throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                              "Unexpected package construction for grid \"" +
                              m_sGridName + "\"."));
    }
  }

  /**
   * Finds a requested grid cell, or creates a new one if it does not
   * already exist.
   * @param iX X grid coordinate.
   * @param iY Y grid coordinate.
   * @param oPlot Plot object.
   * @return GridValue object.
   * @throws ModelException if the coordinates are invalid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected GridValue GetGridValue(int iX, int iY, Plot oPlot) throws
      ModelException {
    GridValue oReturn = null;
    int i;
    for (i = 0; i < mp_oGridVals.size(); i++) {
      oReturn = (GridValue) mp_oGridVals.elementAt(i);
      if (oReturn.GetCell().GetX() == iX && oReturn.GetCell().GetY() == iY) {
        break;
      }
      else {
        oReturn = null;
      }
    }

    if (oReturn == null) {
      int iNumBools, iNumFloats, iNumInts, iNumChars;
      if (mp_sFloatDataMembers == null) {
        iNumFloats = 0;
      }
      else {
        iNumFloats = mp_sFloatDataMembers.length;
      }
      if (mp_sIntDataMembers == null) {
        iNumInts = 0;
      }
      else {
        iNumInts = mp_sIntDataMembers.length;
      }
      if (mp_sCharDataMembers == null) {
        iNumChars = 0;
      }
      else {
        iNumChars = mp_sCharDataMembers.length;
      }
      if (mp_sBoolDataMembers == null) {
        iNumBools = 0;
      }
      else {
        iNumBools = mp_sBoolDataMembers.length;
      }

      //Create a new GridValue
      if (m_fLengthXCells > 0 && m_fLengthYCells > 0) {
        oReturn = new GridValue(iX, iY, iNumFloats, iNumInts, iNumChars,
                                iNumBools, m_fLengthXCells, m_fLengthYCells,
                                oPlot);

      }
      else {
        oReturn = new GridValue(iX, iY, iNumFloats, iNumInts, iNumChars,
                                iNumBools, oPlot);
      }
      mp_oGridVals.add(oReturn);
    }

    return oReturn;
  }

  /**
   * Gets the grid's name.
   * @return The name.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public String GetName() {
    return m_sGridName;
  }

  /**
   * Gets the X cell length for this grid.
   * @return X cell length, in meters.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetXCellLength() {
    return m_fLengthXCells;
  }

  /**
   * Gets the Y cell length for this grid.
   * @return Y cell length, in meters.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public float GetYCellLength() {
    return m_fLengthYCells;
  }

  /**
   * Sets the length of cells in the X direction.  If the length of Y is 0,
   * it will be set to the same value.
   * @param fXCellLength Length of cells in the X direction, in meters.
   * @throws ModelException If the value is negative.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetXCellLength(float fXCellLength) throws ModelException {
    if (fXCellLength < 0) {
      throw(new ModelException(ErrorGUI.BAD_ARGUMENT, "JAVA",
                               "Attempt to set the X cell length of grid " +
                               m_sGridName + " with an invalid value - \"" +
                               fXCellLength + "\"."));
    }

    if (fXCellLength != m_fLengthXCells) {
      m_bEdited = true;
      m_fLengthXCells = fXCellLength;

      if (m_fLengthYCells == 0) {
        m_fLengthYCells = m_fLengthXCells;
      }
    }
  }

  /**
   * Sets the length of cells in the Y direction.  If the length of X is 0,
   * it will be set to the same value.
   * @param fYCellLength Length of cells in the Y direction, in meters.
   * @throws ModelException If the value is negative.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void SetYCellLength(float fYCellLength) throws ModelException {
    if (fYCellLength < 0) {
      throw(new ModelException(ErrorGUI.BAD_ARGUMENT, "JAVA",
                               "Attempt to set the Y cell length of grid " +
                               m_sGridName + " with an invalid value - \"" +
                               fYCellLength + "\"."));
    }

    if (fYCellLength != m_fLengthYCells) {
      m_bEdited = true;
      m_fLengthYCells = fYCellLength;

      if (m_fLengthXCells == 0) {
        m_fLengthXCells = m_fLengthYCells;
      }
    }
  }

  /**
   * Erases all current grid map values without touching any other settings.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void ClearMapValues() {
    mp_oGridVals = null;
    mp_oGridVals = new Vector(0);
  }

  /**
   * Gets the list of data members.
   * @return The list of data members.
   */
  public DataMember[] GetDataMembers() {
    return mp_oDataMembers;
  }

  /**
   * Sets the list of data members. This should ONLY be used to rename
   * data members.
   * @param p_oNewDataMembers The list of data members.
   */
  public void SetDataMembers(DataMember[] p_oNewDataMembers) {
    mp_oDataMembers = p_oNewDataMembers;
  }


  /**
   * Gets the list of package data members.
   * @return The list of package data members, or NULL if there are no separate
   * package data members.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public DataMember[] GetPackageDataMembers() {
    return mp_oPackageDataMembers;
  }

  /**
   * Sets the list of package data members. This should ONLY be used to rename
   * data members.
   * @param p_oNewDataMembers The list of data members.
    */
  public void SetPackageDataMembers(DataMember[] p_oNewDataMembers) {
    mp_oPackageDataMembers = p_oNewDataMembers;
  }


  /**
   * Writes the grid's XML grid map to a file.  If there are no grid map
   * values, this will not write anything.
   * @param oOut File stream to write to.
   * @param oPlot Plot object.
   * @throws ModelException if there is something wrong with the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void WriteXML(BufferedWriter oOut, Plot oPlot) throws ModelException {
    try {
      if (!m_bEdited) {
        return; //nothing to write
      }
      int i;

      //Opening tag
      oOut.write("<grid gridName=\"" + m_sGridName + "\">");

      //Integer settings
      if (mp_sIntDataMembers != null && mp_sIntDataMembers.length > 0) {
        oOut.write("<ma_intCodes>");

        for (i = 0; i < mp_sIntDataMembers.length; i++) {
          oOut.write("<ma_intCode label=\"" + mp_sIntDataMembers[i] +
                     "\">" + i + "</ma_intCode>");
        }
        oOut.write("</ma_intCodes>");
      }

      //Float settings
      if (mp_sFloatDataMembers != null && mp_sFloatDataMembers.length > 0) {
        oOut.write("<ma_floatCodes>");

        for (i = 0; i < mp_sFloatDataMembers.length; i++) {
          oOut.write("<ma_floatCode label=\"" + mp_sFloatDataMembers[i] +
                     "\">" + i + "</ma_floatCode>");
        }
        oOut.write("</ma_floatCodes>");
      }

      //Char settings
      if (mp_sCharDataMembers != null && mp_sCharDataMembers.length > 0) {
        oOut.write("<ma_charCodes>");

        for (i = 0; i < mp_sCharDataMembers.length; i++) {
          oOut.write("<ma_charCode label=\"" + mp_sCharDataMembers[i] +
                     "\">" + i + "</ma_charCode>");
        }
        oOut.write("</ma_charCodes>");
      }

      //Bool settings
      if (mp_sBoolDataMembers != null && mp_sBoolDataMembers.length > 0) {
        oOut.write("<ma_boolCodes>");

        for (i = 0; i < mp_sBoolDataMembers.length; i++) {
          oOut.write("<ma_boolCode label=\"" + mp_sBoolDataMembers[i] +
                     "\">" + i + "</ma_boolCode>");
        }
        oOut.write("</ma_boolCodes>");
      }

      //Package integer settings
      if (mp_sPackageIntDataMembers != null &&
          mp_sPackageIntDataMembers.length > 0) {
        oOut.write("<ma_packageIntCodes>");

        for (i = 0; i < mp_sPackageIntDataMembers.length; i++) {
          oOut.write("<ma_intCode label=\"" + mp_sPackageIntDataMembers[i] +
                     "\">" + i + "</ma_intCode>");
        }
        oOut.write("</ma_packageIntCodes>");
      }

      //Package float settings
      if (mp_sPackageFloatDataMembers != null &&
          mp_sPackageFloatDataMembers.length > 0) {
        oOut.write("<ma_packageFloatCodes>");

        for (i = 0; i < mp_sPackageFloatDataMembers.length; i++) {
          oOut.write("<ma_floatCode label=\"" + mp_sPackageFloatDataMembers[i] +
                     "\">" + i + "</ma_floatCode>");
        }
        oOut.write("</ma_packageFloatCodes>");
      }

      //Package char settings
      if (mp_sPackageCharDataMembers != null &&
          mp_sPackageCharDataMembers.length > 0) {
        oOut.write("<ma_packageCharCodes>");

        for (i = 0; i < mp_sPackageCharDataMembers.length; i++) {
          oOut.write("<ma_charCode label=\"" + mp_sPackageCharDataMembers[i] +
                     "\">" + i + "</ma_charCode>");
        }
        oOut.write("</ma_packageCharCodes>");
      }

      //Package bool settings
      if (mp_sPackageBoolDataMembers != null &&
          mp_sPackageBoolDataMembers.length > 0) {
        oOut.write("<ma_packageBoolCodes>");

        for (i = 0; i < mp_sPackageBoolDataMembers.length; i++) {
          oOut.write("<ma_boolCode label=\"" + mp_sPackageBoolDataMembers[i] +
                     "\">" + i + "</ma_boolCode>");
        }
        oOut.write("</ma_packageBoolCodes>");
      }

      //X plot length
      oOut.write("<ma_plotLenX>" + String.valueOf(oPlot.GetPlotXLength()) +
                 "</ma_plotLenX>");

      //Y plot length
      oOut.write("<ma_plotLenY>" + String.valueOf(oPlot.GetPlotYLength()) +
                 "</ma_plotLenY>");

      //X cell length, if available
      if (m_fLengthXCells > 0) {
        oOut.write("<ma_lengthXCells>" + String.valueOf(m_fLengthXCells) +
                   "</ma_lengthXCells>");
      }

      //Y cell length, if available
      if (m_fLengthYCells > 0) {
        oOut.write("<ma_lengthYCells>" + String.valueOf(m_fLengthYCells) +
                   "</ma_lengthYCells>");
      }

      //Individual values
      GridValue oVal;
      for (i = 0; i < mp_oGridVals.size(); i++) {
        oVal = (GridValue) mp_oGridVals.elementAt(i);

        oVal.WriteXML(oOut);
      }

      //Closing grid tag
      oOut.write("</grid>");
    }
    catch (java.io.IOException e) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "JAVA",
      "There was a problem writing the parameter file."));
    }

  }
}

/**
 * Class for holding the data for a single grid cell.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
class GridValue
    extends DataMemberData {

  /**Grid cell*/
  protected Cell m_oCell;

  /**Vector of PackageGridValue objects.*/
  protected Vector mp_oPackages = new Vector(0);

  /**
   * Constructor.
   * @param iX X cell coordinate.
   * @param iY Y cell coordinate.
   * @param iNumFloats Number of float data members.
   * @param iNumInts Number of integer data members.
   * @param iNumChars Number of char data members.
   * @param iNumBools Number of bool data members.
   * @param oPlot Plot object.
   * @throws ModelException if cell coordinates are invalid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  GridValue(int iX, int iY, int iNumFloats, int iNumInts, int iNumChars,
            int iNumBools, Plot oPlot) throws ModelException {
    super(iNumFloats, iNumInts, iNumChars, iNumBools);

    m_oCell = new Cell(iX, iY, oPlot);
  }

  /**
   * Constructor.
   * @param iX X cell coordinate.
   * @param iY Y cell coordinate.
   * @param iNumFloats Number of float data members.
   * @param iNumInts Number of integer data members.
   * @param iNumChars Number of char data members.
   * @param iNumBools Number of bool data members.
   * @param fLengthXCells Length of cell in X direction.
   * @param fLengthYCells Length of cell in Y direction.
   * @param oPlot Plot object.
   * @throws ModelException if cell coordinates are invalid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  GridValue(int iX, int iY, int iNumFloats, int iNumInts, int iNumChars,
            int iNumBools, float fLengthXCells, float fLengthYCells,
            Plot oPlot) throws ModelException {
    super(iNumFloats, iNumInts, iNumChars, iNumBools);

    m_oCell = new Cell(iX, iY, fLengthXCells, fLengthYCells, oPlot);
  }

  /**Gets the cell for this object.
   * @return Cell object.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public Cell GetCell() {
    return m_oCell;
  }

  /**
   * Adds a package to this grid cell.
   * @param oPackage Package to add.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void AddPackage(PackageGridValue oPackage) {
    mp_oPackages.add(oPackage);
  }

  /**
   * Writes this grid's data to XML.
   * @param oOut File to write to.
   * @throws java.io.IOException if there is a problem writing to file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void WriteXML(BufferedWriter oOut) throws java.io.IOException {
    PackageGridValue oPackage;
    int i;

    //Opening grid cell tag
    oOut.write("<ma_v x=\"" + String.valueOf(m_oCell.GetX()) +
               "\" y=\"" + String.valueOf(m_oCell.GetY()) + "\">");

    //Write the ints
    for (i = 0; i < mp_iInts.length; i++) {
      if (mp_iInts[i] != null) {
        oOut.write("<int c=\"" + String.valueOf(i) + "\">" +
                   mp_iInts[i].toString() + "</int>");
      }
    }

    //Write the floats
    for (i = 0; i < mp_fFloats.length; i++) {
      if (mp_fFloats[i] != null) {
        oOut.write("<fl c=\"" + String.valueOf(i) + "\">" +
                   mp_fFloats[i].toString() + "</fl>");
      }
    }

    //Write the chars
    for (i = 0; i < mp_sChars.length; i++) {
      if (mp_sChars[i] != null) {
        oOut.write("<ch c=\"" + String.valueOf(i) + "\">" +
                   mp_sChars[i] + "</ch>");
      }
    }

    //Write the bools
    for (i = 0; i < mp_bBools.length; i++) {
      if (mp_bBools[i] != null) {
        oOut.write("<bl c=\"" + String.valueOf(i) + "\">" +
                   mp_bBools[i].toString() + "</bl>");
      }
    }

    //Write the packages, if any
    for (i = 0; i < mp_oPackages.size(); i++) {
      oPackage = (PackageGridValue) mp_oPackages.elementAt(i);
      oPackage.WriteXML(oOut);
    }

    oOut.write("</ma_v>");
  }
}

/**
 * Class for holding the data for a single package.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
class PackageGridValue
    extends DataMemberData {

  /**
   * Constructor.
   * @param iNumFloats Number of float package data members.
   * @param iNumInts Number of integer package data members.
   * @param iNumChars Number of char package data members.
   * @param iNumBools Number of bool package data members.
   * @throws ModelException if cell coordinates are invalid.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  PackageGridValue(int iNumFloats, int iNumInts, int iNumChars,
                   int iNumBools) throws ModelException {
    super(iNumFloats, iNumInts, iNumChars, iNumBools);
  }

  /**
   * Writes this package's data to XML.
   * @param oOut File to write to.
   * @throws java.io.IOException if there is a problem writing to file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void WriteXML(BufferedWriter oOut) throws java.io.IOException {
    int i;

    //Opening package tag
    oOut.write("<pkg>");

    //Write the ints
    for (i = 0; i < mp_iInts.length; i++) {
      if (mp_iInts[i] != null) {
        oOut.write("<pint c=\"" + String.valueOf(i) + "\">" +
                   mp_iInts[i].toString() + "</pint>");
      }
    }

    //Write the floats
    for (i = 0; i < mp_fFloats.length; i++) {
      if (mp_fFloats[i] != null) {
        oOut.write("<pfl c=\"" + String.valueOf(i) + "\">" +
                   mp_fFloats[i].toString() + "</pfl>");
      }
    }

    //Write the chars
    for (i = 0; i < mp_sChars.length; i++) {
      if (mp_sChars[i] != null) {
        oOut.write("<pch c=\"" + String.valueOf(i) + "\">" +
                   mp_sChars[i] + "</pch>");
      }
    }

    //Write the bools
    for (i = 0; i < mp_bBools.length; i++) {
      if (mp_bBools[i] != null) {
        oOut.write("<pbl c=\"" + String.valueOf(i) + "\">" +
                   mp_bBools[i].toString() + "</pbl>");
      }
    }

    //Closing package tag
    oOut.write("</pkg>");
  }
}
