package javawrapper;

import java.io.FileWriter;
import java.io.File;

/**
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestSeedPredationBehaviors
    extends ModelTestCase {

  /**
   * Tests that the correct parameters are displayed for each behavior.
   */
  public void testFormatDataForDisplay() {
    try {
      GUIManager oManager = new GUIManager(null);
      SeedPredationBehaviors oPredation = null;
      TreePopulation oPop = null;
      String[] p_sExpected;
      Behavior oBeh;

      //Case:  no seed predation behaviors are enabled.
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      assertEquals(null, oPredation.FormatDataForDisplay(oPop));

      //Case:  only functional response seed predation is enabled.
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oPredation.GetBehaviorByKey("funcResponsePredation");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          oPop));
      p_sExpected = new String[] {
          "Predator Initial Density - num/sq m",
          "Max Decline Rate in Absence of Food, predators/week",
          "Predator Population Demographic Efficiency",
          "Density-Dependent Coefficient",
          "Max Intake Rate - seeds per predator per day",
          "Foraging Efficiency",
          "Proportion of Seeds Removed Each Week During Germination Period",
          "Number of Weeks in Which Seedfall Occurs",
          "Weeks to Run Seed Predation Model (1 - 52)",
          "Week During Predation Model Run Germination Begins",
          "Seed Predation Output Filename, If Desired",
          "Preserve Predator Densities Between SORTIE Timesteps",
      };
      TestTable(oPredation.FormatDataForDisplay(oPop), p_sExpected);
            
      //Case:  only neighborhood seed predation is enabled
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      oBeh = oPredation.GetBehaviorByKey("Neighborhood Predation");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, TreePopulation.SEED,
          oPop));
      p_sExpected = new String[] {
      		"Neighborhood Predation - Masting \"A\"",
          "Neighborhood Predation - Masting \"B\"",
          "Neighborhood Predation - Masting \"C\"",
          "Neighborhood Predation - Non-Masting \"A\"",
          "Neighborhood Predation - Non-Masting \"B\"",
          "Neighborhood Predation - Non-Masting \"C\"",
          "Neighborhood Predation - Masting Load Factor",
          "Neighborhood Predation - Non-Masting Load Factor",
          "Neighborhood Predation - Masting Seed Density, #/m2/yr",
          "Neighborhood Predation - Minimum Neighbor DBH (cm)",
          "Neighborhood Predation - Neighbor Search Radius (m)",    
      };
      TestTable(oPredation.FormatDataForDisplay(oPop), p_sExpected);
       
      //Case:  all are enabled.
      oManager.InputXMLParameterFile(WriteNoSeedPredationXMLFile());
      oPredation = oManager.GetSeedPredationBehaviors();
      oPop = oManager.GetTreePopulation();
      int i;
      for (i = 0; i < oPredation.mp_oChildBehaviors.length; i++) {
        oPredation.mp_oChildBehaviors[i].m_bIsEnabled = true;
        oPredation.mp_oChildBehaviors[i].AddSpeciesTypeCombo(new
            SpeciesTypeCombo(0, TreePopulation.SEED,
                             oPop));
      }
      p_sExpected = new String[] {
      		"Predator Initial Density - num/sq m",
          "Max Decline Rate in Absence of Food, predators/week",
          "Predator Population Demographic Efficiency",
          "Density-Dependent Coefficient",
          "Max Intake Rate - seeds per predator per day",
          "Foraging Efficiency",
          "Proportion of Seeds Removed Each Week During Germination Period",
          "Number of Weeks in Which Seedfall Occurs",
          "Weeks to Run Seed Predation Model (1 - 52)",
          "Week During Predation Model Run Germination Begins",
          "Seed Predation Output Filename, If Desired",
          "Preserve Predator Densities Between SORTIE Timesteps",
          "Neighborhood Predation - Masting \"A\"",
          "Neighborhood Predation - Masting \"B\"",
          "Neighborhood Predation - Masting \"C\"",
          "Neighborhood Predation - Non-Masting \"A\"",
          "Neighborhood Predation - Non-Masting \"B\"",
          "Neighborhood Predation - Non-Masting \"C\"",
          "Neighborhood Predation - Masting Load Factor",
          "Neighborhood Predation - Non-Masting Load Factor",
          "Neighborhood Predation - Masting Seed Density, #/m2/yr",
          "Neighborhood Predation - Minimum Neighbor DBH (cm)",
          "Neighborhood Predation - Neighbor Search Radius (m)",
      };
      TestTable(oPredation.FormatDataForDisplay(oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for seed predation.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Makes sure all parameters read correctly.
   */
  public void testParFileReading() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Valid file 1
      oManager.ClearCurrentData();
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      SeedPredationBehaviors oPred = oManager.GetSeedPredationBehaviors();
      double SMALL_VAL = 0.00001;
      
      assertEquals(oPred.m_fFuncRespPredInitDensity.GetValue(), 0.46875, SMALL_VAL);
      assertEquals(oPred.m_fFuncRespMaxInstDeclineRate.GetValue(), -0.050622442, SMALL_VAL);
      assertEquals(oPred.m_fFuncRespDemogEfficiency.GetValue(), 0.154258, SMALL_VAL);
      assertEquals(oPred.m_fFunRespDensDepCoeff.GetValue(), -0.718667, SMALL_VAL);
      assertEquals(oPred.m_fFuncRespMaxIntake.GetValue(), 1042.2, 0.01);
      assertEquals(oPred.m_fFuncRespForagingEff.GetValue(), 0.001389, SMALL_VAL);
      assertEquals(oPred.m_iFuncRespNumWeeksSeedFall.GetValue(), 12);
      assertEquals(oPred.m_iFuncRespNumWeeksToModel.GetValue(), 52);
      assertEquals(oPred.m_iFuncRespWeekGermStarts.GetValue(), 32);
      assertEquals(oPred.m_iPreservePredatorDensities.GetValue(), 0);
      assertEquals(oPred.m_fFuncRespPropGerm.GetValue(), 0.35, SMALL_VAL);
      assertEquals(oPred.m_sFuncRespOutputFilename.GetValue(), "TestSeeds.txt");
      assertEquals(oPred.m_fNeighPredMastingDensity.GetValue(), 4, SMALL_VAL);
      assertEquals(oPred.m_fNeighPredMinDbh.GetValue(), 10, SMALL_VAL);
      assertEquals(oPred.m_fNeighPredRadius.GetValue(), 15, SMALL_VAL);

      assertEquals( ( (Float) oPred.mp_fNeighPredMastingA.GetValue().elementAt(0)).
                   floatValue(), 0.1, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingA.GetValue().elementAt(1)).
                   floatValue(), 0.25, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingB.GetValue().elementAt(0)).
          floatValue(), 2, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingB.GetValue().elementAt(1)).
          floatValue(), 0.3, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingC.GetValue().elementAt(0)).
          floatValue(), 1, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingC.GetValue().elementAt(1)).
          floatValue(), -0.2, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingA.GetValue().elementAt(0)).
          floatValue(), 0.5, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingA.GetValue().elementAt(1)).
          floatValue(), 0.6, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingB.GetValue().elementAt(0)).
          floatValue(), 10, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingB.GetValue().elementAt(1)).
          floatValue(), 0.9, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingC.GetValue().elementAt(0)).
          floatValue(), 0.2, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingC.GetValue().elementAt(1)).
          floatValue(), 1.4, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingLoad.GetValue().elementAt(0)).
          floatValue(), 1, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredMastingLoad.GetValue().elementAt(1)).
          floatValue(), 2, SMALL_VAL);
      
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingLoad.GetValue().elementAt(0)).
          floatValue(), 0.2, SMALL_VAL);
      assertEquals( ( (Float) oPred.mp_fNeighPredNonMastingLoad.GetValue().elementAt(1)).
          floatValue(), 0.4, SMALL_VAL);
    }
    catch (ModelException oErr) {
      fail("Establishment validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
    	new File(sFileName).delete();
    }
  }
    
  /**
   * Tests ValidateData().
   * @throws ModelException if testing fails
   */
  public void testValidateData() throws ModelException {
    GUIManager oManager = null;
    String sFileName = null;
    SeedPredationBehaviors oPred = null;
    try {
      try {

        oManager = new GUIManager(null);

        //Valid file 1
        oManager.ClearCurrentData();
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oManager.GetSeedPredationBehaviors().ValidateData(oManager.
            GetTreePopulation());
      }
      catch (ModelException oErr) {
        fail("Seed predation validation failed with message " + oErr.getMessage());
      }

      //Exception processing - functional response predation is enabled but
      //no disperse behaviors are
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.GetBehaviorByKey("Neighborhood Predation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("Neighborhood Predation").ClearSpeciesTypeCombos();
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            m_bIsEnabled = false;
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            ClearSpeciesTypeCombos();
        oPred.ValidateData(oManager.GetTreePopulation());
        fail(
            "Seed predation didn't catch no disperse with functional response" +
            " predation.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());

      //Exception processing - neighborhood predation is enabled but
      //no disperse behaviors are
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.GetBehaviorByKey("funcResponsePredation").m_bIsEnabled = false;
        oPred.GetBehaviorByKey("funcResponsePredation").ClearSpeciesTypeCombos();
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            m_bIsEnabled = false;
        oManager.GetDisperseBehaviors().GetBehaviorByKey("non spatial disperse").
            ClearSpeciesTypeCombos();
        oPred.ValidateData(oManager.GetTreePopulation());
        fail(
            "Seed predation didn't catch no disperse with functional response" +
            " predation.");
      }
      catch (ModelException oErr) {
        ;
      }
      
      //Exception processing - case: number of weeks of seedfall is less than
      //or equal to zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespNumWeeksSeedFall.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for number of weeks of" +
        		"seedfall.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Initial predator density is less than or equal to zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iPreservePredatorDensities.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for initial predator " +
        		"densities.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Number of weeks to run the model is less than 0
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespNumWeeksToModel.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for number of weeks to " +
        		"run the model.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Number of weeks to run the model is greater than 52
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespNumWeeksToModel.SetValue(63);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for number of weeks to " +
        		"run the model.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Week to start germination is less than 0
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespWeekGermStarts.SetValue(-3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for week germination " +
        		"starts.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Week to start germination is greater than 52
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_iFuncRespWeekGermStarts.SetValue(63);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for week germination " +
        		"starts.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Proportion germinating is not a proportion
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fFuncRespPropGerm.SetValue((float)1.3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for proportion germinating.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Neighborhood predation minimum neighbor 
      //DBH is less than zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fNeighPredMinDbh.SetValue((float)-1.3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for minimum neighbor DBH.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Neighborhood predation neighbor radius is
      //less than zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fNeighPredRadius.SetValue((float)-1.3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for neighborhood radius.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());
      
      //Exception processing - case: Neighborhood predation masting seed
      //density threshold is less than zero
      try {
        sFileName = WriteValidFile1();
        oManager.InputXMLParameterFile(sFileName);
        oPred = oManager.GetSeedPredationBehaviors();
        oPred.m_fNeighPredMastingDensity.SetValue((float)-1.3);
        oPred.ValidateData(oManager.GetTreePopulation());
        fail("Seed predation didn't catch bad value for masting density" +
        		" threshold.");
      }
      catch (ModelException oErr) {
        ;
      }
      //Reset
      sFileName = WriteValidFile1();
      oManager.InputXMLParameterFile(sFileName);
      oPred = oManager.GetSeedPredationBehaviors();
      oPred.ValidateData(oManager.GetTreePopulation());  
      
    } catch (ModelException oErr) {
    	fail("Seed predation validation failed with message " + oErr.getMessage());
    } catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }
    
  /**
   * Writes a valid seed predation parameter file with all the behaviors.
   * @throws IOException if there is a problem writing the file.
   * @return String Filename.
   */
  private String WriteValidFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">"); 
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>96</plot_lenX>");
    oOut.write("<plot_lenY>96</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\"/>");
    oOut.write("<tr_species speciesName=\"Species_2\"/>");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_maxSeedlingHeight>");
    oOut.write("<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>");
    oOut.write("</tr_maxSeedlingHeight>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.54</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0614</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.5944</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_interceptDiam10ToDBH>");
    oOut.write("<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>");
    oOut.write("</tr_interceptDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.368</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.0269</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0241</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("<tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>");
    oOut.write("</tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_whatSaplingHeightDiam>");
    oOut.write("<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>");
    oOut.write("</tr_whatSaplingHeightDiam>");
    oOut.write("<tr_whatAdultHeightDiam>");
    oOut.write("<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>");
    oOut.write("</tr_whatAdultHeightDiam>");
    oOut.write("<tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>");
    oOut.write("</tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>");
    oOut.write("</tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>");
    oOut.write("</tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>");
    oOut.write("</tr_whatSaplingCrownHeightHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>non spatial disperse</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>"); 
    oOut.write("<behaviorName>functional response seed predation</behaviorName>");
    oOut.write("<version>1</version>"); 
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>"); 
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Neighborhood Seed Predation</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<disperse>");
    oOut.write("<di_minDbhForReproduction>");
    oOut.write("<di_mdfrVal species=\"Species_1\">15.0</di_mdfrVal>");
    oOut.write("<di_mdfrVal species=\"Species_2\">15.0</di_mdfrVal>");
    oOut.write("</di_minDbhForReproduction>");
    oOut.write("<di_nonSpatialSlopeOfLambda>");
    oOut.write("<di_nssolVal species=\"Species_2\">0</di_nssolVal>");
    oOut.write("<di_nssolVal species=\"Species_1\">0</di_nssolVal>");
    oOut.write("</di_nonSpatialSlopeOfLambda>");
    oOut.write("<di_nonSpatialInterceptOfLambda>");
    oOut.write("<di_nsiolVal species=\"Species_1\">1</di_nsiolVal>");
    oOut.write("<di_nsiolVal species=\"Species_2\">1</di_nsiolVal>");
    oOut.write("</di_nonSpatialInterceptOfLambda>");
    oOut.write("<di_seedDistributionMethod>0</di_seedDistributionMethod>");
    oOut.write("</disperse>");
    oOut.write("<seedPredation>");
    oOut.write("<pr_predatorInitialDensity>0.46875</pr_predatorInitialDensity>");
    oOut.write("<pr_maxInstantaneousDeclineRate>-0.050622442</pr_maxInstantaneousDeclineRate>");
    oOut.write("<pr_demographicEfficiency>0.154258</pr_demographicEfficiency>");
    oOut.write("<pr_densityDependentCoefficient>-0.718667</pr_densityDependentCoefficient>");
    oOut.write("<pr_maxIntakeRate>1042.2</pr_maxIntakeRate>"); 
    oOut.write("<pr_foragingEfficiency>0.001389</pr_foragingEfficiency>");
    oOut.write("<pr_numWeeksSeedFall>12</pr_numWeeksSeedFall>"); 
    oOut.write("<pr_numWeeksToModel>52</pr_numWeeksToModel>");
    oOut.write("<pr_weekGerminationStarts>32</pr_weekGerminationStarts>");
    oOut.write("<pr_preservePredatorDensities>0</pr_preservePredatorDensities>");
    oOut.write("<pr_proportionGerminating>0.35</pr_proportionGerminating>");
    oOut.write("<pr_outputFilename>TestSeeds.txt</pr_outputFilename>");
    oOut.write("<pr_neighPredMastingDensity>4</pr_neighPredMastingDensity>");
    oOut.write("<pr_neighPredMinNeighDBH>10</pr_neighPredMinNeighDBH>");
    oOut.write("<pr_neighPredRadius>15</pr_neighPredRadius>");
    oOut.write("<pr_neighPredMastingA>");
    oOut.write("<pr_npmaVal species=\"Species_1\">0.1</pr_npmaVal>");
    oOut.write("<pr_npmaVal species=\"Species_2\">0.25</pr_npmaVal>");
    oOut.write("</pr_neighPredMastingA>");
    oOut.write("<pr_neighPredMastingB>");
    oOut.write("<pr_npmbVal species=\"Species_1\">2</pr_npmbVal>");
    oOut.write("<pr_npmbVal species=\"Species_2\">0.3</pr_npmbVal>");
    oOut.write("</pr_neighPredMastingB>");
    oOut.write("<pr_neighPredMastingC>");
    oOut.write("<pr_npmcVal species=\"Species_1\">1</pr_npmcVal>");
    oOut.write("<pr_npmcVal species=\"Species_2\">-0.2</pr_npmcVal>");
    oOut.write("</pr_neighPredMastingC>");
    oOut.write("<pr_neighPredNonMastingA>");
    oOut.write("<pr_npnmaVal species=\"Species_1\">0.5</pr_npnmaVal>");
    oOut.write("<pr_npnmaVal species=\"Species_2\">0.6</pr_npnmaVal>");
    oOut.write("</pr_neighPredNonMastingA>");
    oOut.write("<pr_neighPredNonMastingB>");
    oOut.write("<pr_npnmbVal species=\"Species_1\">10</pr_npnmbVal>");
    oOut.write("<pr_npnmbVal species=\"Species_2\">0.9</pr_npnmbVal>");
    oOut.write("</pr_neighPredNonMastingB>");
    oOut.write("<pr_neighPredNonMastingC>");
    oOut.write("<pr_npnmcVal species=\"Species_1\">0.2</pr_npnmcVal>");
    oOut.write("<pr_npnmcVal species=\"Species_2\">1.4</pr_npnmcVal>");
    oOut.write("</pr_neighPredNonMastingC>");
    oOut.write("<pr_neighPredMastingPCALoadFactor>");
    oOut.write("<pr_npmplfVal species=\"Species_1\">1</pr_npmplfVal>");
    oOut.write("<pr_npmplfVal species=\"Species_2\">2</pr_npmplfVal>");
    oOut.write("</pr_neighPredMastingPCALoadFactor>");
    oOut.write("<pr_neighPredNonMastingPCALoadFactor>");
    oOut.write("<pr_npnmplfVal species=\"Species_1\">0.2</pr_npnmplfVal>");
    oOut.write("<pr_npnmplfVal species=\"Species_2\">0.4</pr_npnmplfVal>");
    oOut.write("</pr_neighPredNonMastingPCALoadFactor>");
    oOut.write("</seedPredation>");
    oOut.write("</paramFile>");
    oOut.close();
    return sFileName;
  }
    
  /**
   * Writes a file with no seed predation settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>August 18, 2006: Created (LEM)
   */
  protected String WriteNoSeedPredationXMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>50</timesteps>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>0.0</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">40.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.7</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.7</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.7</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.7</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Constant GLI</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}