package javawrapper;

import junit.framework.*;
import java.io.FileWriter;
import java.io.File;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.InputSource;

/**
 * Tests the TreePopulation class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestTreePopulation
    extends TestCase {

  /**
   * Tests the ValidateData method.
   */
  public void testValidateData() {
    String sFileName = "";
    GUIManager oManager;
    TreePopulation oPop;
    try {
      //Neutral case
      oManager = new GUIManager(null);
      sFileName = WriteXMLTreeMap1();
      oManager.InputXMLFile(sFileName, null);
      oPop = oManager.GetTreePopulation();
      oPop.ValidateData(oPop);

      //Set up seedling initial densities
      oPop.m_fSeedlingHeightClass1.SetValue((float)10.0);
      oPop.m_fSeedlingHeightClass2.SetValue((float)100.0);
      Float[] p_fDensities = new Float[]{new Float(0), new Float(0.1), new Float(0.2), new Float(0.4), new Float(0), new Float(0), new Float(0), new Float(0), new Float(0)};
      oPop.SetVectorValues(oPop.mp_fSeedlingClass1Density, p_fDensities);
      oPop.ValidateData(oPop);
    } catch (ModelException oErr) {
        fail("Test ValidateData failed with message " + oErr.getMessage());
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Test the case where there is data for seedling densities of size class
      //1, but the size class bounds fields are both 0
      oManager = new GUIManager(null);
      sFileName = WriteXMLTreeMap1();
      oManager.InputXMLFile(sFileName, null);
      oPop = oManager.GetTreePopulation();
      oPop.ValidateData(oPop);

      oPop.m_fSeedlingHeightClass1.SetValue((float)0.0);
      oPop.m_fSeedlingHeightClass2.SetValue((float)0.0);
      Float[] p_fDensities = new Float[]{new Float(0), new Float(0.1), new Float(0.2), new Float(0.4), new Float(0), new Float(0), new Float(0), new Float(0), new Float(0)};
      oPop.SetVectorValues(oPop.mp_fSeedlingClass1Density, p_fDensities);
      oPop.ValidateData(oPop);
      fail("Test ValidateData didn't catch lack of seedling bounds.");
    } catch (ModelException oErr) {
       ;
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Test the case where there is data for seedling densities of size class
      //2, but the size class bounds fields are both 0
      oManager = new GUIManager(null);
      sFileName = WriteXMLTreeMap1();
      oManager.InputXMLFile(sFileName, null);
      oPop = oManager.GetTreePopulation();
      oPop.ValidateData(oPop);

      oPop.m_fSeedlingHeightClass1.SetValue((float)0.0);
      oPop.m_fSeedlingHeightClass2.SetValue((float)0.0);
      Float[] p_fDensities = new Float[]{new Float(0), new Float(0.1), new Float(0.2), new Float(0.4), new Float(0), new Float(0), new Float(0), new Float(0), new Float(0)};
      oPop.SetVectorValues(oPop.mp_fSeedlingClass2Density, p_fDensities);
      oPop.ValidateData(oPop);
      fail("Test ValidateData didn't catch lack of seedling bounds.");
    } catch (ModelException oErr) {
       ;
    }
    finally {
      new File(sFileName).delete();
    }

    try {
      //Test the case where there is data for seedling densities of size class
      //3, but the size class bounds fields are both 0
      oManager = new GUIManager(null);
      sFileName = WriteXMLTreeMap1();
      oManager.InputXMLFile(sFileName, null);
      oPop = oManager.GetTreePopulation();
      oPop.ValidateData(oPop);

      oPop.m_fSeedlingHeightClass1.SetValue((float)0.0);
      oPop.m_fSeedlingHeightClass2.SetValue((float)0.0);
      Float[] p_fDensities = new Float[]{new Float(0), new Float(0.1), new Float(0.2), new Float(0.4), new Float(0), new Float(0), new Float(0), new Float(0), new Float(0)};
      oPop.SetVectorValues(oPop.mp_fSeedlingClass3Density, p_fDensities);
      oPop.ValidateData(oPop);
      fail("Test ValidateData didn't catch lack of seedling bounds.");
    } catch (ModelException oErr) {
       ;
    }
    finally {
      new File(sFileName).delete();
    }
  }

  /**
   * Tests reading a tab delimited tree map.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   * <br>March 28, 2005:  Updated with new tree map format (LEM)
   */
  public void testTabDelimTreeMapReading() {
    try {
      GUIManager oManager = new GUIManager(null);
      oManager.InputXMLFile(WriteTreeMapParFile(), null);
      TreePopulation oPop = oManager.GetTreePopulation();

      /************************
       Map file 1 - no trees
       *************************/
      String sFileName = WriteTabTreeMap1();
      oPop.AddTabDelimTreeMapFile(null, sFileName);

      //Make sure the right number of trees were read
      assertEquals(0, oPop.mp_oTrees.size());
      new File(sFileName).delete();

      /************************
        Map file 2 - one tree
       *************************/
      sFileName = WriteTabTreeMap2();
      oPop.AddTabDelimTreeMapFile(null, sFileName);

      //Make sure the right number of trees were read
      assertEquals(1, oPop.mp_oTrees.size());
      //Verify tree
      Tree oTree = (Tree) oPop.mp_oTrees.elementAt(0);
      int iSpecies = oTree.m_iSpecies,
          iType = oTree.m_iType;
      assertEquals(2, iSpecies);
      assertEquals(TreePopulation.ADULT, iType);
      assertEquals( (float) 5.58472,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 9.69238,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 14.06372,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);
      oPop.ClearTrees();

      new File(sFileName).delete();

      /************************
        Map file 3 - several trees
       *************************/
      sFileName = WriteTabTreeMap3();
      oPop.AddTabDelimTreeMapFile(null, sFileName);

      //Make sure the right number of trees were read
      assertEquals(7, oPop.mp_oTrees.size());
      //Verify trees
      //Tree 1
      oTree = (Tree) oPop.mp_oTrees.elementAt(0);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(0, iSpecies);
      assertEquals(TreePopulation.SEEDLING, iType);
      assertEquals( (float) 5.58472,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 9.69238,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 0.12,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Diam10")].floatValue(),
                   0.001);
      assertEquals( (float) 1.2,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")].floatValue(),
                   0.001);

      //Tree 2
      oTree = (Tree) oPop.mp_oTrees.elementAt(1);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(1, iSpecies);
      assertEquals(TreePopulation.SAPLING, iType);
      assertEquals( (float) 4.01001,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 10.553,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 8.50342,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //Tree 3
      oTree = (Tree) oPop.mp_oTrees.elementAt(2);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(2, iSpecies);
      assertEquals(TreePopulation.ADULT, iType);
      assertEquals( (float) 7.47681,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 29.0466,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 33.36319,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertEquals( (float) 5.05853,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")].floatValue(),
                   0.001);

      //Tree 4
      oTree = (Tree) oPop.mp_oTrees.elementAt(3);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(0, iSpecies);
      assertEquals(TreePopulation.SNAG, iType);
      assertEquals( (float) 4.93164,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 20.4529,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 13.21426,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //Tree 5
      oTree = (Tree) oPop.mp_oTrees.elementAt(4);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(1, iSpecies);
      assertEquals(TreePopulation.SAPLING, iType);
      assertEquals( (float) 9.11865,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 33.8013,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 2.77977,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertEquals( (float) 5.12386,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")].floatValue(),
                   0.001);

      //Tree 6
      oTree = (Tree) oPop.mp_oTrees.elementAt(5);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(2, iSpecies);
      assertEquals(TreePopulation.ADULT, iType);
      assertEquals( (float) 9.31396,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 41.3818,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 39.6077,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //Tree 7
      oTree = (Tree) oPop.mp_oTrees.elementAt(6);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(0, iSpecies);
      assertEquals(TreePopulation.SNAG, iType);
      assertEquals( (float) 3.92456,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 45.7397,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 7.78549,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertEquals( (float) 8.53078,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")].floatValue(),
                   0.001);

      oPop.ClearTrees();

      new File(sFileName).delete();

      /************************
       Map file 4 - several trees, including some outside the plot;
       for this, the user is warned and does not elect to continue
       *************************/
      sFileName = WriteTabTreeMap4();
      //Tell the tester to accept the warning about the trees outside of the
      //plot
      javax.swing.JOptionPane.showMessageDialog(null,
                                                "You will be warned ONE TIME about trees outside the plot.\n" +
                                                "Please tell it NOT to continue.",
                                                "TESTING MESSAGE",
                                                javax.swing.JOptionPane.
                                                INFORMATION_MESSAGE);
      oPop.AddTabDelimTreeMapFile(null, sFileName);

      //Make sure the right number of trees were read
      assertEquals(0, oPop.mp_oTrees.size());

      new File(sFileName).delete();

      /************************
       Map file 4 - several trees, including some outside the plot;
       for this, the user is warned and elects to continue
       *************************/
      sFileName = WriteTabTreeMap4();
      //Tell the tester to accept the warning about the trees outside of the
      //plot
      javax.swing.JOptionPane.showMessageDialog(null,
                                                "You will be warned ONE TIME about trees outside the plot.\n" +
                                                "Please tell it to continue.",
                                                "TESTING MESSAGE",
                                                javax.swing.JOptionPane.
                                                INFORMATION_MESSAGE);
      oPop.AddTabDelimTreeMapFile(null, sFileName);

      //Make sure the right number of trees were read
      assertEquals(2, oPop.mp_oTrees.size());
      //Verify trees
      //Tree 1
      oTree = (Tree) oPop.mp_oTrees.elementAt(0);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(0, iSpecies);
      assertEquals(TreePopulation.ADULT, iType);
      assertEquals( (float) 9.33838,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 99.5117,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 28.63892,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //Tree 2
      oTree = (Tree) oPop.mp_oTrees.elementAt(1);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(1, iSpecies);
      assertEquals(TreePopulation.ADULT, iType);
      assertEquals( (float) 8.80127,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 94.9768,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 29.42276,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //oPop.ClearTrees(); DON'T CLEAR THIS TIME

      new File(sFileName).delete();

      /************************
       Read in another map file while there are trees; user elects to
       replace those trees
       *************************/
      sFileName = WriteTabTreeMap3();
      //Tell the tester to replace trees
      javax.swing.JOptionPane.showMessageDialog(null,
                                                "Please tell SORTIE to replace trees in the next dialog.",
                                                "TESTING MESSAGE",
                                                javax.swing.JOptionPane.
                                                INFORMATION_MESSAGE);
      oPop.AddTabDelimTreeMapFile(null, sFileName);

      //Make sure the right number of trees were read
      assertEquals(7, oPop.mp_oTrees.size());
      //Verify trees
      //Tree 1
      oTree = (Tree) oPop.mp_oTrees.elementAt(0);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(0, iSpecies);
      assertEquals(TreePopulation.SEEDLING, iType);
      assertEquals( (float) 5.58472,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 9.69238,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 0.12,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Diam10")].floatValue(),
                   0.001);
      assertEquals( (float) 1.2,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")].floatValue(),
                   0.001);

      //Tree 2
      oTree = (Tree) oPop.mp_oTrees.elementAt(1);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(1, iSpecies);
      assertEquals(TreePopulation.SAPLING, iType);
      assertEquals( (float) 4.01001,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 10.553,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 8.50342,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //Tree 3
      oTree = (Tree) oPop.mp_oTrees.elementAt(2);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(2, iSpecies);
      assertEquals(TreePopulation.ADULT, iType);
      assertEquals( (float) 7.47681,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 29.0466,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 33.36319,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertEquals( (float) 5.05853,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")].floatValue(),
                   0.001);

      //Tree 4
      oTree = (Tree) oPop.mp_oTrees.elementAt(3);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(0, iSpecies);
      assertEquals(TreePopulation.SNAG, iType);
      assertEquals( (float) 4.93164,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 20.4529,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 13.21426,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //Tree 5
      oTree = (Tree) oPop.mp_oTrees.elementAt(4);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(1, iSpecies);
      assertEquals(TreePopulation.SAPLING, iType);
      assertEquals( (float) 9.11865,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 33.8013,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 2.77977,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertEquals( (float) 5.12386,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")].floatValue(),
                   0.001);

      //Tree 6
      oTree = (Tree) oPop.mp_oTrees.elementAt(5);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(2, iSpecies);
      assertEquals(TreePopulation.ADULT, iType);
      assertEquals( (float) 9.31396,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 41.3818,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 39.6077,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //Tree 7
      oTree = (Tree) oPop.mp_oTrees.elementAt(6);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(0, iSpecies);
      assertEquals(TreePopulation.SNAG, iType);
      assertEquals( (float) 3.92456,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 45.7397,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 7.78549,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertEquals( (float) 8.53078,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")].floatValue(),
                   0.001);

      //oPop.ClearTrees(); DON'T CLEAR TREES

      new File(sFileName).delete();

      /************************
       Read in another map file while there are trees; user elects to
       add to those trees
       *************************/
      sFileName = WriteTabTreeMap4();
      //Tell the tester to accept the warning about the trees outside of the
      //plot
      javax.swing.JOptionPane.showMessageDialog(null,
                                                "You will be warned ONE TIME about trees outside the plot.\n" +
                                                "Please tell it to continue.\n" +
                                                "Also, ADD TO existing trees.",
                                                "TESTING MESSAGE",
                                                javax.swing.JOptionPane.
                                                INFORMATION_MESSAGE);
      oPop.AddTabDelimTreeMapFile(null, sFileName);

      //Make sure the right number of trees were read
      assertEquals(9, oPop.mp_oTrees.size());
      //Verify trees
      //Tree 1
      oTree = (Tree) oPop.mp_oTrees.elementAt(7);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(0, iSpecies);
      assertEquals(TreePopulation.ADULT, iType);
      assertEquals( (float) 9.33838,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 99.5117,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 28.63892,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //Tree 2
      oTree = (Tree) oPop.mp_oTrees.elementAt(8);
      iSpecies = oTree.m_iSpecies;
      iType = oTree.m_iType;
      assertEquals(1, iSpecies);
      assertEquals(TreePopulation.ADULT, iType);
      assertEquals( (float) 8.80127,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "X")].floatValue(), 0.001);
      assertEquals( (float) 94.9768,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Y")].floatValue(), 0.001);
      assertEquals( (float) 29.42276,
                   oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "DBH")].floatValue(),
                   0.001);
      assertNull(oTree.mp_fFloats[oPop.GetCodeForDataMember(oPop.
          mp_sTreeFloatDataMembers[iSpecies][iType], "Height")]);

      //oPop.ClearTrees(); DON'T CLEAR THIS TIME

      new File(sFileName).delete();

      /************************
       Error processing - invalid species
       *************************/
      try {
        sFileName = WriteTabTreeMap5();
        //Tell the user to add to existing trees
        javax.swing.JOptionPane.showMessageDialog(null,
                                                "Please tell it to ADD TO existing trees.",
                                                "TESTING MESSAGE",
                                                javax.swing.JOptionPane.
                                                INFORMATION_MESSAGE);
        oPop.AddTabDelimTreeMapFile(null, sFileName);
        fail("Didn't catch unrecognized species in tab-delimited tree map testing.");
      }
      catch (ModelException oErr) {
        //Make sure that no trees were added
        assertEquals(9, oPop.mp_oTrees.size());
        //Make sure the error was right
        assertTrue(oErr.getMessage().indexOf("invalid species") > 0);
        oPop.ClearTrees();
      }
      finally {
        new File(sFileName).delete();
      }

      /************************
       Error processing - invalid life history stage
       *************************/
      try {
        sFileName = WriteTabTreeMap6();
        oPop.AddTabDelimTreeMapFile(null, sFileName);
        fail("Didn't catch unrecognized life history stage in tab-delimited tree map testing.");
      }
      catch (ModelException oErr) {
        //Make sure that no trees were added
        assertEquals(0, oPop.mp_oTrees.size());
        //Make sure the error was right
        assertTrue(oErr.getMessage().indexOf("invalid tree type") > 0);
      }
      finally {
        new File(sFileName).delete();
      }

      /************************
       Error processing - negative diameter
       *************************/
      try {
        sFileName = WriteTabTreeMap7();
        oPop.AddTabDelimTreeMapFile(null, sFileName);
        fail("Didn't catch negative diameter in tab-delimited tree map testing.");
      }
      catch (ModelException oErr) {
        //Make sure that no trees were added
        assertEquals(0, oPop.mp_oTrees.size());
        //Make sure the error was right
        assertTrue(oErr.getMessage().indexOf("invalid tree diameter") > 0);
      }
      finally {
        new File(sFileName).delete();
      }

      /************************
       Error processing - negative height
       *************************/
      try {
        sFileName = WriteTabTreeMap8();
        oPop.AddTabDelimTreeMapFile(null, sFileName);
        fail("Didn't catch negative height in tab-delimited tree map testing.");
      }
      catch (ModelException oErr) {
        //Make sure that no trees were added
        assertEquals(0, oPop.mp_oTrees.size());
        //Make sure the error was right
        assertTrue(oErr.getMessage().indexOf("invalid tree height") > 0);
      }
      finally {
        new File(sFileName).delete();
      }

      /************************
       Error processing - bad format
       *************************/
      try {
        sFileName = WriteTabTreeMap9();
        oPop.AddTabDelimTreeMapFile(null, sFileName);
        fail("Didn't catch bad format in tab-delimited tree map testing.");
      }
      catch (ModelException oErr) {
        //Make sure that no trees were added
        assertEquals(0, oPop.mp_oTrees.size());
        //Make sure the error was right
        assertTrue(oErr.getMessage().indexOf("invalid format") > 0);
      }
      finally {
        new File(sFileName).delete();
      }

      System.out.println("Tab-delimited tree map reading test succeeded.");
    }
    catch (ModelException oErr) {
      fail("Tab-delimited tree map reading failed with message " +
           oErr.getMessage());
    }
    catch (java.io.IOException oErr) {
      fail("IO failure for tab-delimited tree map reading.  Message: " +
           oErr.getMessage());
    }
  }

  /**
   * Tests reading multiple Tree maps.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  /*  public void testComboMapReading() {
      GUIManager oManager = null;
      try {
        oManager = new GUIManager(null);
        oManager.ClearCurrentData();

        //Set up and parse our test file
        //Create our parser
        XMLReader oParser = XMLReaderFactory.createXMLReader(
            "org.apache.xerces.parsers.SAXParser");
        //Create the handler that looks for setup data and register it
        DefaultHandler oHandler = new ParameterFileParser(oManager);
        oParser.setContentHandler(oHandler);
        //Parse the file
        String sFileName = WriteXMLTreeMap1();
        InputSource oToParse = new InputSource(sFileName);
        oParser.parse(oToParse);

        //Enable an absolute growth behavior so there will be suppression/release
        //data read in
   oManager.GetGrowthBehaviors().GetBehaviorByKey("absba").m_bIsEnabled = true;

        TreePopulation oPop = oManager.GetTreePopulation();

        //Verify - total number of trees
        assertEquals(8, oPop.mp_oTrees.size());

        //Validate each Tree
        VerifyXMLTreeMap(oPop, 0);
        new File(sFileName).delete();

        //Now read in the regular Tree map
        sFileName = WriteV5TreeMap();
        oPop.AddOldTreeMapFile(sFileName, false);

        //Verify total number of trees
        assertEquals(18, oPop.mp_oTrees.size());

        //Verify each Tree
        VerifyV5TreeMap(oPop, 8);
        new File(sFileName).delete();

        //Fake rundata file
        oHandler = null;
        oHandler = new ParameterFileParser(oManager);
        oParser.setContentHandler(oHandler);
        sFileName = WriteXMLTreeMap2();
        oToParse = new InputSource(sFileName);
        oParser.parse(oToParse);

        assertEquals(26, oPop.mp_oTrees.size());
        VerifyXMLTreeMap(oPop, 18);

        new File(sFileName).delete();

        System.out.println("Multiple Tree map reading test succeeded.");
      }
      catch (ModelException oErr) {
        fail("XML Tree map reading failed with message " + oErr.getMessage());
      }
      catch (org.xml.sax.SAXException oE) {
        fail("Parse test could not parse parameter file - message: " +
             oE.getMessage());
      }
      catch (java.io.IOException oE) {
        fail("Caught IOException.  Message: " + oE.getMessage());
      }
    } */

  /**
   * Tests the Tree class.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void testTreeClass() {
    TreePopulation oPop = new TreePopulation(null);

    try {
      oPop.SetSpeciesNames(new String[] {"ACSA", "ACRU", "QURU"});

      //Normal processing
      Tree oTree = new Tree(0, 2, 5, 4, 3, 1, oPop);
      assertEquals(2, oTree.m_iSpecies);
      assertEquals(0, oTree.m_iType);
      assertEquals(5, oTree.mp_fFloats.length);
      assertEquals(4, oTree.mp_iInts.length);
      assertEquals(3, oTree.mp_sChars.length);
      assertEquals(1, oTree.mp_bBools.length);

      int i;
      for (i = 0; i < oTree.mp_fFloats.length; i++) {
        assertTrue(oTree.mp_fFloats[i] == null);
      }

      for (i = 0; i < oTree.mp_iInts.length; i++) {
        assertTrue(oTree.mp_iInts[i] == null);
      }

      for (i = 0; i < oTree.mp_sChars.length; i++) {
        assertTrue(oTree.mp_sChars[i] == null);
      }

      for (i = 0; i < oTree.mp_bBools.length; i++) {
        assertTrue(oTree.mp_bBools[i] == null);
      }

      oTree.SetValue(0, new Boolean(true));
      oTree.SetValue(1, new String("test"));
      oTree.SetValue(2, new Integer(1));
      oTree.SetValue(3, new Float(2.3));

      assertTrue(oTree.mp_bBools[0].equals(new Boolean(true)));
      assertTrue(oTree.mp_sChars[1].equals("test"));
      assertTrue(oTree.mp_iInts[2].equals(new Integer(1)));
      assertTrue(oTree.mp_fFloats[3].equals(new Float(2.3)));

      System.out.println("Normal processing for Tree succeeded.");
    }
    catch (ModelException oErr) {
      fail("Tree testing failed with message " + oErr.getMessage());
    }

    //Exception processing

    try {
      //Bad species
      new Tree(2, -2, 5, 4, 3, 1, oPop);
      fail("Tree constructor failed to catch bad species.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad species
      new Tree(2, 12, 5, 4, 3, 1, oPop);
      fail("Tree constructor failed to catch bad species.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad type
      new Tree( -1, 2, 5, 4, 3, 1, oPop);
      fail("Tree constructor failed to catch bad type.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad type
      new Tree(11, 2, 5, 4, 3, 1, oPop);
      fail("Tree constructor failed to catch bad type.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad float
      Tree oTree = new Tree(1, 2, 5, 4, 3, 1, oPop);
      oTree.SetValue( -1, new Float(2));
      fail("Tree constructor failed to catch bad float index.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad float
      Tree oTree = new Tree(1, 2, 5, 6, 6, 6, oPop);
      oTree.SetValue(5, new Float(2));
      fail("Tree constructor failed to catch bad float index.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad int
      Tree oTree = new Tree(1, 2, 5, 4, 3, 1, oPop);
      oTree.SetValue( -1, new Integer(2));
      fail("Tree constructor failed to catch bad int index.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad int
      Tree oTree = new Tree(1, 2, 6, 5, 6, 6, oPop);
      oTree.SetValue(5, new Integer(2));
      fail("Tree constructor failed to catch bad int index.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad char
      Tree oTree = new Tree(1, 2, 5, 4, 3, 1, oPop);
      oTree.SetValue( -1, new String("2"));
      fail("Tree constructor failed to catch bad char index.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad char
      Tree oTree = new Tree(1, 2, 6, 6, 5, 6, oPop);
      oTree.SetValue(5, new String("2"));
      fail("Tree constructor failed to catch bad char index.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad bool
      Tree oTree = new Tree(1, 2, 5, 4, 3, 1, oPop);
      oTree.SetValue( -1, new Boolean(true));
      fail("Tree constructor failed to catch bad bool index.");
    }
    catch (ModelException oErr) {
      ;
    }

    try {
      //Bad bool
      Tree oTree = new Tree(1, 2, 6, 6, 6, 5, oPop);
      oTree.SetValue(5, new Boolean(true));
      fail("Tree constructor failed to catch bad bool index.");
    }
    catch (ModelException oErr) {
      ;
    }
  }

  /**
   * Tests reading XML Tree maps.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  public void testXMLTreeMapReading() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();

      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      sFileName = WriteXMLTreeMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);

      TreePopulation oPop = oManager.GetTreePopulation();

      //Verify - total number of trees
      assertEquals(8, oPop.mp_oTrees.size());

      //Validate each Tree
      VerifyXMLTreeMap(oPop, 0);

      new File(sFileName).delete();
      System.out.println(
          "XML Tree map reading normal processing test succeeded.");
    }
    catch (ModelException oErr) {
      fail("XML Tree map reading failed with message " + oErr.getMessage());
    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    try {
      //Exception processing
      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
    }
    catch (ModelException oErr) {
      fail("XML Tree map reading failed with message " + oErr.getMessage());
    }

    try {
      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      sFileName = WriteXMLTreeMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
    }
    catch (ModelException oErr) {
      fail("XML Tree map reading failed with message " + oErr.getMessage());

    }
    catch (org.xml.sax.SAXException oE) {
      fail("Parse test could not parse parameter file - message: " +
           oE.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    try {
      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      sFileName = WriteErrorXMLTreeMap1();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("Did not catch X value bigger than plot.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    try {
      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      sFileName = WriteErrorXMLTreeMap2();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("Did not catch negative X value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    try {
      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      sFileName = WriteErrorXMLTreeMap3();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("Did not catch Y value bigger than plot.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    try {
      //Set up and parse our test file
      //Create our parser
      XMLReader oParser = XMLReaderFactory.createXMLReader(
          "org.apache.xerces.parsers.SAXParser");
      //Create the handler that looks for setup data and register it
      DefaultHandler oHandler = new ParameterFileParser(oManager);
      oParser.setContentHandler(oHandler);
      //Parse the file
      sFileName = WriteErrorXMLTreeMap4();
      InputSource oToParse = new InputSource(sFileName);
      oParser.parse(oToParse);
      fail("Did not catch negative Y value.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (org.xml.sax.SAXException oE) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    new File(sFileName).delete();
    System.out.println(
        "XML Tree map reading exception processing test succeeded.");

  }

  /**
   * Verifies the trees added by the XML Tree map.
   * @param oPop TreePopulation object containing the trees.
   * @param iStartIndex The index at which to start verifying the trees.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected void VerifyXMLTreeMap(TreePopulation oPop, int iStartIndex) {
    //Tree 1
    Tree oTree = (Tree) oPop.mp_oTrees.elementAt(iStartIndex);
    assertEquals(0, oTree.m_iSpecies);
    assertEquals(2, oTree.m_iType);
    assertTrue(4 <= oTree.mp_fFloats.length);
    assertTrue(3 <= oTree.mp_iInts.length);
    assertTrue(2 <= oTree.mp_sChars.length);
    assertTrue(2 <= oTree.mp_bBools.length);
    assertEquals(0.981445,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "X")].
                 floatValue(), 0.001);
    assertEquals(41.499001,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "Y")].
                 floatValue(), 0.001);
    assertEquals(89.027000,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "DBH")].
                 floatValue(), 0.001);
    assertEquals(1.1,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "float 1")].
                 floatValue(), 0.001);
    assertEquals(1, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 1")].
                 intValue());
    assertEquals(2, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 2")].
                 intValue());
    assertEquals(3, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 3")].
                 intValue());
    assertEquals("char val 1", oTree.mp_sChars[oPop.GetCodeForDataMember(
        oPop.mp_sTreeCharDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "char 1")]);
    assertEquals("char val 2", oTree.mp_sChars[oPop.GetCodeForDataMember(
        oPop.mp_sTreeCharDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "char 2")]);
    assertEquals(true, oTree.mp_bBools[oPop.GetCodeForDataMember(
        oPop.mp_sTreeBoolDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "bool 1")].booleanValue());
    assertEquals(false, oTree.mp_bBools[oPop.GetCodeForDataMember(
        oPop.mp_sTreeBoolDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "bool 2")].booleanValue());

    //Tree 2
    oTree = (Tree) oPop.mp_oTrees.elementAt(iStartIndex + 1);
    assertEquals(1, oTree.m_iSpecies);
    assertEquals(2, oTree.m_iType);
    assertTrue(4 <= oTree.mp_fFloats.length);
    assertTrue(3 <= oTree.mp_iInts.length);
    assertTrue(2 <= oTree.mp_sChars.length);
    assertTrue(2 <= oTree.mp_bBools.length);
    assertEquals(3.149410,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "X")].
                 floatValue(), 0.001);
    assertEquals(11.313500,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "Y")].
                 floatValue(), 0.001);
    assertEquals(89.988998,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "DBH")].
                 floatValue(), 0.001);
    assertEquals(2.1,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "float 1")].
                 floatValue(), 0.001);
    assertEquals(4, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 1")].
                 intValue());
    assertEquals(5, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 2")].
                 intValue());
    assertEquals(6, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 3")].
                 intValue());
    assertEquals("char val 2.1", oTree.mp_sChars[oPop.GetCodeForDataMember(
        oPop.mp_sTreeCharDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "char 1")]);
    assertEquals("char val 2.2", oTree.mp_sChars[oPop.GetCodeForDataMember(
        oPop.mp_sTreeCharDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "char 2")]);
    assertEquals(true, oTree.mp_bBools[oPop.GetCodeForDataMember(
        oPop.mp_sTreeBoolDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "bool 1")].booleanValue());
    assertEquals(false, oTree.mp_bBools[oPop.GetCodeForDataMember(
        oPop.mp_sTreeBoolDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "bool 2")].booleanValue());

    //Tree 3
    oTree = (Tree) oPop.mp_oTrees.elementAt(iStartIndex + 2);
    assertEquals(2, oTree.m_iSpecies);
    assertEquals(2, oTree.m_iType);
    assertTrue(4 <= oTree.mp_fFloats.length);
    assertTrue(3 <= oTree.mp_iInts.length);
    assertTrue(2 <= oTree.mp_sChars.length);
    assertTrue(2 <= oTree.mp_bBools.length);
    assertEquals(10.981445,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "X")].
                 floatValue(), 0.001);
    assertEquals(51.499001,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "Y")].
                 floatValue(), 0.001);
    assertEquals(9.027000,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "DBH")].
                 floatValue(), 0.001);
    assertEquals(3.1,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "float 1")].
                 floatValue(), 0.001);
    assertEquals(7, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 1")].
                 intValue());
    assertEquals(8, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 2")].
                 intValue());
    assertEquals(9, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 3")].
                 intValue());
    assertEquals("char val 3.1", oTree.mp_sChars[oPop.GetCodeForDataMember(
        oPop.mp_sTreeCharDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "char 1")]);
    assertEquals("char val 3.2", oTree.mp_sChars[oPop.GetCodeForDataMember(
        oPop.mp_sTreeCharDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "char 2")]);
    assertEquals(false, oTree.mp_bBools[oPop.GetCodeForDataMember(
        oPop.mp_sTreeBoolDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "bool 1")].booleanValue());
    assertEquals(true, oTree.mp_bBools[oPop.GetCodeForDataMember(
        oPop.mp_sTreeBoolDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "bool 2")].booleanValue());

    //Tree 4
    oTree = (Tree) oPop.mp_oTrees.elementAt(iStartIndex + 3);
    assertEquals(1, oTree.m_iSpecies);
    assertEquals(1, oTree.m_iType);
    assertTrue(3 <= oTree.mp_fFloats.length);
    assertTrue(1 <= oTree.mp_iInts.length);
    assertTrue(1 <= oTree.mp_sChars.length);
    assertTrue(1 <= oTree.mp_bBools.length);
    assertEquals(1.923830,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "X")].
                 floatValue(), 0.001);
    assertEquals(55.781200,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "Y")].
                 floatValue(), 0.001);
    assertEquals(43.942001,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "Height")].
                 floatValue(), 0.001);
    assertEquals(10, oTree.mp_iInts[oPop.GetCodeForDataMember(
        oPop.mp_sTreeIntDataMembers[oTree.m_iSpecies][oTree.m_iType], "int 1")].
                 intValue());
    assertEquals("char val 4.1", oTree.mp_sChars[oPop.GetCodeForDataMember(
        oPop.mp_sTreeCharDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "char 1")]);
    assertEquals(false, oTree.mp_bBools[oPop.GetCodeForDataMember(
        oPop.mp_sTreeBoolDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "bool 1")].booleanValue());

    //Tree 5
    oTree = (Tree) oPop.mp_oTrees.elementAt(iStartIndex + 4);
    assertEquals(2, oTree.m_iSpecies);
    assertEquals(1, oTree.m_iType);
    assertTrue(3 <= oTree.mp_fFloats.length);
    assertTrue(0 <= oTree.mp_iInts.length);
    assertTrue(0 <= oTree.mp_sChars.length);
    assertTrue(0 <= oTree.mp_bBools.length);
    assertEquals(0.810547,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "X")].
                 floatValue(), 0.001);
    assertEquals(54.584999,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "Y")].
                 floatValue(), 0.001);
    assertEquals(81.760002,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType],
        "Height")].
                 floatValue(), 0.001);

    //Tree 6
    oTree = (Tree) oPop.mp_oTrees.elementAt(iStartIndex + 5);
    assertEquals(2, oTree.m_iSpecies);
    assertEquals(3, oTree.m_iType);
    assertTrue(3 <= oTree.mp_fFloats.length);
    assertTrue(0 <= oTree.mp_iInts.length);
    assertTrue(0 <= oTree.mp_sChars.length);
    assertTrue(0 <= oTree.mp_bBools.length);
    assertEquals(7.470700,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "X")].
                 floatValue(), 0.001);
    assertEquals(93.603500,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "Y")].
                 floatValue(), 0.001);
    assertEquals(41.171001,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "DBH")].
                 floatValue(), 0.001);

    //Tree 7
    oTree = (Tree) oPop.mp_oTrees.elementAt(iStartIndex + 6);
    assertEquals(1, oTree.m_iSpecies);
    assertEquals(3, oTree.m_iType);
    assertTrue(3 <= oTree.mp_fFloats.length);
    assertTrue(0 <= oTree.mp_iInts.length);
    assertTrue(0 <= oTree.mp_sChars.length);
    assertTrue(0 <= oTree.mp_bBools.length);
    assertEquals(6.528320,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "X")].
                 floatValue(), 0.001);
    assertEquals(102.182999,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "Y")].
                 floatValue(), 0.001);
    assertEquals(43.853001,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "DBH")].
                 floatValue(), 0.001);

    //Tree 8
    oTree = (Tree) oPop.mp_oTrees.elementAt(iStartIndex + 7);
    assertEquals(0, oTree.m_iSpecies);
    assertEquals(3, oTree.m_iType);
    assertTrue(3 <= oTree.mp_fFloats.length);
    assertTrue(0 <= oTree.mp_iInts.length);
    assertTrue(0 <= oTree.mp_sChars.length);
    assertTrue(0 <= oTree.mp_bBools.length);
    assertEquals(0.732422,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "X")].
                 floatValue(), 0.001);
    assertEquals(99.418900,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "Y")].
                 floatValue(), 0.001);
    assertEquals(83.000999,
                 oTree.mp_fFloats[oPop.GetCodeForDataMember(
        oPop.mp_sTreeFloatDataMembers[oTree.m_iSpecies][oTree.m_iType], "DBH")].
                 floatValue(), 0.001);

  }

  /**
   * Writes a map to create trees.  This file has no trees.
   * @return Name of file.
   * @throws IOException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 28, 2005: Created (LEM)
   */
  protected String WriteTabTreeMap1() throws java.io.IOException {
    String sFileName = "\\testfile.txt";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X\tY\tSpecies\tType\tDiam\tHeight");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a map to create trees.  This file has one tree.
   * @return Name of file.
   * @throws IOException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 28, 2005: Created (LEM)
   */
  protected String WriteTabTreeMap2() throws java.io.IOException {
    String sFileName = "\\testfile.txt";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X\tY\tSpecies\tType\tDiam\tHeight\n");
    oOut.write("5.58472\t9.69238\tBeech\tAdult\t14.06372\t0\n");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a map to create trees.  This file has several trees.
   * @return Name of file.
   * @throws IOException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 28, 2005: Created (LEM)
   */
  protected String WriteTabTreeMap3() throws java.io.IOException {
    String sFileName = "\\testfile.txt";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X\tY\tSpecies\tType\tDiam\tHeight\n");
    oOut.write("5.58472\t9.69238\tRed Maple\tSeedling\t0.12\t1.2\n");
    oOut.write("4.01001\t10.553\tOak\tSapling\t8.50342\t0\n");
    oOut.write("7.47681\t29.0466\tBeech\tADULT\t33.36319\t5.05853\n");
    oOut.write("4.93164\t20.4529\tRed Maple\tSNAG\t13.21426\t0\n");
    oOut.write("9.11865\t33.8013\tOak\tsapling\t2.77977\t5.12386\n");
    oOut.write("9.31396\t41.3818\tBeech\taduLT\t39.6077\t0\n");
    oOut.write("3.92456\t45.7397\tRed Maple\tsnag\t7.78549\t8.53078\n");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a map to create trees.  This file has several trees, including
   * some outside the plot.
   * @return Name of file.
   * @throws IOException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 28, 2005: Created (LEM)
   */
  protected String WriteTabTreeMap4() throws java.io.IOException {
    String sFileName = "\\testfile.txt";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X\tY\tSpecies\tType\tDiam\tHeight\n");
    oOut.write("9.33838\t99.5117\tRed Maple\tAdult\t28.63892\t0\n");
    oOut.write("8.80127\t94.9768\tOak\tAdult\t29.42276\t0\n");
    oOut.write("104.865\t5.16968\tBeech\tAdult\t25.93903\t0\n");
    oOut.write("4.32129\t159.867\tRed Maple\tAdult\t14.06372\t0\n");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a map with an error.  This file has an unrecognized species.
   * @return Name of file.
   * @throws IOException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 28, 2005: Created (LEM)
   */
  protected String WriteTabTreeMap5() throws java.io.IOException {
    String sFileName = "\\testfile.txt";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X\tY\tSpecies\tType\tDiam\tHeight\n");
    oOut.write("5.58472\t9.69238\tRed Maple\tSeedling\t0.12\t1.2\n");
    oOut.write("4.01001\t10.553\tOak\tSapling\t8.50342\t0\n");
    oOut.write("7.47681\t29.0466\tCherry\tADULT\t33.36319\t5.05853\n");
    oOut.write("4.93164\t20.4529\tRed Maple\tSNAG\t13.21426\t0\n");
    oOut.write("9.11865\t33.8013\tOak\tsapling\t2.77977\t5.12386\n");
    oOut.write("9.31396\t41.3818\tBeech\taduLT\t39.6077\t0\n");
    oOut.write("3.92456\t45.7397\tRed Maple\tsnag\t7.78549\t8.53078\n");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a map with an error.  This file has an unrecognized life history
   * stage.
   * @return Name of file.
   * @throws IOException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 28, 2005: Created (LEM)
   */
  protected String WriteTabTreeMap6() throws java.io.IOException {
    String sFileName = "\\testfile.txt";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X\tY\tSpecies\tType\tDiam\tHeight\n");
    oOut.write("5.58472\t9.69238\tRed Maple\tSeedling\t0.12\t1.2\n");
    oOut.write("4.01001\t10.553\tOak\tSapling\t8.50342\t0\n");
    oOut.write("7.47681\t29.0466\tBeech\tCoffee_Table\t33.36319\t5.05853\n");
    oOut.write("4.93164\t20.4529\tRed Maple\tSNAG\t13.21426\t0\n");
    oOut.write("9.11865\t33.8013\tOak\tsapling\t2.77977\t5.12386\n");
    oOut.write("9.31396\t41.3818\tBeech\taduLT\t39.6077\t0\n");
    oOut.write("3.92456\t45.7397\tRed Maple\tsnag\t7.78549\t8.53078\n");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a map with an error.  This file has a negative diameter.
   * @return Name of file.
   * @throws IOException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 28, 2005: Created (LEM)
   */
  protected String WriteTabTreeMap7() throws java.io.IOException {
    String sFileName = "\\testfile.txt";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X\tY\tSpecies\tType\tDiam\tHeight\n");
    oOut.write("5.58472\t9.69238\tRed Maple\tSeedling\t0.12\t1.2\n");
    oOut.write("4.01001\t10.553\tOak\tSapling\t8.50342\t0\n");
    oOut.write("7.47681\t29.0466\tBeech\tADULT\t33.36319\t5.05853\n");
    oOut.write("4.93164\t20.4529\tRed Maple\tSNAG\t-13.21426\t0\n");
    oOut.write("9.11865\t33.8013\tOak\tsapling\t2.77977\t5.12386\n");
    oOut.write("9.31396\t41.3818\tBeech\taduLT\t39.6077\t0\n");
    oOut.write("3.92456\t45.7397\tRed Maple\tsnag\t7.78549\t8.53078\n");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a map with an error.  This file has a negative height.
   * @return Name of file.
   * @throws IOException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 28, 2005: Created (LEM)
   */
  protected String WriteTabTreeMap8() throws java.io.IOException {
    String sFileName = "\\testfile.txt";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X\tY\tSpecies\tType\tDiam\tHeight\n");
    oOut.write("5.58472\t9.69238\tRed Maple\tSeedling\t0.12\t1.2\n");
    oOut.write("4.01001\t10.553\tOak\tSapling\t8.50342\t0\n");
    oOut.write("7.47681\t29.0466\tBeech\tADULT\t33.36319\t5.05853\n");
    oOut.write("4.93164\t20.4529\tRed Maple\tSNAG\t13.21426\t0\n");
    oOut.write("9.11865\t33.8013\tOak\tsapling\t2.77977\t-5.12386\n");
    oOut.write("9.31396\t41.3818\tBeech\taduLT\t39.6077\t0\n");
    oOut.write("3.92456\t45.7397\tRed Maple\tsnag\t7.78549\t8.53078\n");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a map with an error.  This file has an invalid format.
   * @return Name of file.
   * @throws IOException if there is an IO problem.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>March 28, 2005: Created (LEM)
   */
  protected String WriteTabTreeMap9() throws java.io.IOException {
    String sFileName = "\\testfile.txt";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("200 200 9 0\n");
    oOut.write("0 0 C:\\SORTIE\\Documentation\\SORTIEBC.par\n");
    oOut.write("5.58472\t9.69238\t5.12222\t6.87094\t4\t0\t0\t0\n");
    oOut.write("4.01001\t10.553\t8.50342\t11.0904\t7\t0\t0\t0\n");
    oOut.write("7.47681\t29.0466\t3.36319\t5.05853\t4\t0\t0\t0\n");
    oOut.write("4.93164\t20.4529\t3.21426\t4.84396\t0\t0\t0\t0\n");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a Tree map file.  There are all four kinds of data members.  This
   * is actually a parameter file with a Tree map in it.
   * @return The file name.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected String WriteXMLTreeMap1() throws ModelException {
    try {
      String sFileName = "\\testtreemap1.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<paramFile fileCode=\"06010101\">");

      //Plot - so we can test reading an old tree map
      oOut.write("<plot>");
      oOut.write("<plot_lenX>200.0</plot_lenX>");
      oOut.write("<plot_lenY>200.0</plot_lenY>");
      oOut.write("<plot_latitude>55.37</plot_latitude>");
      oOut.write("</plot>");

      oOut.write("<trees>");
      oOut.write("<tr_speciesList>");
      oOut.write("<tr_species speciesName=\"ACSA\"/>");
      oOut.write("<tr_species speciesName=\"ACRU\"/>");
      oOut.write("<tr_species speciesName=\"QURU\"/>");
      oOut.write("<tr_species speciesName=\"PRSE\"/>");
      oOut.write("<tr_species speciesName=\"PIST\"/>");
      oOut.write("<tr_species speciesName=\"TSCA\"/>");
      oOut.write("<tr_species speciesName=\"FRAM\"/>");
      oOut.write("<tr_species speciesName=\"BEAL\"/>");
      oOut.write("<tr_species speciesName=\"FAGR\"/>");
      oOut.write("</tr_speciesList>");
      oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
      oOut.write("<tr_canopyHeight>");
      oOut.write("<tr_chVal species=\"ACSA\">39.48</tr_chVal>");
      oOut.write("<tr_chVal species=\"ACRU\">39.54</tr_chVal>");
      oOut.write("<tr_chVal species=\"QURU\">40.0</tr_chVal>");
      oOut.write("<tr_chVal species=\"PRSE\">40.0</tr_chVal>");
      oOut.write("<tr_chVal species=\"PIST\">45.0</tr_chVal>");
      oOut.write("<tr_chVal species=\"TSCA\">40.0</tr_chVal>");
      oOut.write("<tr_chVal species=\"FRAM\">39.14</tr_chVal>");
      oOut.write("<tr_chVal species=\"BEAL\">39.47</tr_chVal>");
      oOut.write("<tr_chVal species=\"FAGR\">33.18</tr_chVal>");
      oOut.write("</tr_canopyHeight>");
      oOut.write("<tr_stdAsympCrownRad>");
      oOut.write("<tr_sacrVal species=\"ACSA\">0.0549</tr_sacrVal>");
      oOut.write("<tr_sacrVal species=\"ACRU\">0.0614</tr_sacrVal>");
      oOut.write("<tr_sacrVal species=\"QURU\">0.0242</tr_sacrVal>");
      oOut.write("<tr_sacrVal species=\"PRSE\">0.0251</tr_sacrVal>");
      oOut.write("<tr_sacrVal species=\"PIST\">0.0239</tr_sacrVal>");
      oOut.write("<tr_sacrVal species=\"TSCA\">0.0303</tr_sacrVal>");
      oOut.write("<tr_sacrVal species=\"FRAM\">0.0328</tr_sacrVal>");
      oOut.write("<tr_sacrVal species=\"BEAL\">0.0247</tr_sacrVal>");
      oOut.write("<tr_sacrVal species=\"FAGR\">0.0484</tr_sacrVal>");
      oOut.write("</tr_stdAsympCrownRad>");
      oOut.write("<tr_stdCrownRadExp>");
      oOut.write("<tr_screVal species=\"ACSA\">1.0</tr_screVal>");
      oOut.write("<tr_screVal species=\"ACRU\">1.0</tr_screVal>");
      oOut.write("<tr_screVal species=\"QURU\">1.0</tr_screVal>");
      oOut.write("<tr_screVal species=\"PRSE\">1.0</tr_screVal>");
      oOut.write("<tr_screVal species=\"PIST\">1.0</tr_screVal>");
      oOut.write("<tr_screVal species=\"TSCA\">1.0</tr_screVal>");
      oOut.write("<tr_screVal species=\"FRAM\">1.0</tr_screVal>");
      oOut.write("<tr_screVal species=\"BEAL\">1.0</tr_screVal>");
      oOut.write("<tr_screVal species=\"FAGR\">1.0</tr_screVal>");
      oOut.write("</tr_stdCrownRadExp>");
      oOut.write("<tr_conversionDiam10ToDBH>");
      oOut.write("<tr_cdtdVal species=\"ACSA\">0.8008</tr_cdtdVal>");
      oOut.write("<tr_cdtdVal species=\"ACRU\">0.5944</tr_cdtdVal>");
      oOut.write("<tr_cdtdVal species=\"QURU\">0.7059</tr_cdtdVal>");
      oOut.write("<tr_cdtdVal species=\"PRSE\">0.7776</tr_cdtdVal>");
      oOut.write("<tr_cdtdVal species=\"PIST\">0.6933</tr_cdtdVal>");
      oOut.write("<tr_cdtdVal species=\"TSCA\">0.8014</tr_cdtdVal>");
      oOut.write("<tr_cdtdVal species=\"FRAM\">0.7992</tr_cdtdVal>");
      oOut.write("<tr_cdtdVal species=\"BEAL\">0.7926</tr_cdtdVal>");
      oOut.write("<tr_cdtdVal species=\"FAGR\">0.7803</tr_cdtdVal>");
      oOut.write("</tr_conversionDiam10ToDBH>");
      oOut.write("<tr_stdAsympCrownHt>");
      oOut.write("<tr_sachVal species=\"ACSA\">0.389</tr_sachVal>");
      oOut.write("<tr_sachVal species=\"ACRU\">0.368</tr_sachVal>");
      oOut.write("<tr_sachVal species=\"QURU\">0.464</tr_sachVal>");
      oOut.write("<tr_sachVal species=\"PRSE\">0.454</tr_sachVal>");
      oOut.write("<tr_sachVal species=\"PIST\">0.405</tr_sachVal>");
      oOut.write("<tr_sachVal species=\"TSCA\">0.201</tr_sachVal>");
      oOut.write("<tr_sachVal species=\"FRAM\">0.301</tr_sachVal>");
      oOut.write("<tr_sachVal species=\"BEAL\">0.42</tr_sachVal>");
      oOut.write("<tr_sachVal species=\"FAGR\">0.315</tr_sachVal>");
      oOut.write("</tr_stdAsympCrownHt>");
      oOut.write("<tr_stdCrownHtExp>");
      oOut.write("<tr_scheVal species=\"ACSA\">1.0</tr_scheVal>");
      oOut.write("<tr_scheVal species=\"ACRU\">1.0</tr_scheVal>");
      oOut.write("<tr_scheVal species=\"QURU\">1.0</tr_scheVal>");
      oOut.write("<tr_scheVal species=\"PRSE\">1.0</tr_scheVal>");
      oOut.write("<tr_scheVal species=\"PIST\">1.0</tr_scheVal>");
      oOut.write("<tr_scheVal species=\"TSCA\">1.0</tr_scheVal>");
      oOut.write("<tr_scheVal species=\"FRAM\">1.0</tr_scheVal>");
      oOut.write("<tr_scheVal species=\"BEAL\">1.0</tr_scheVal>");
      oOut.write("<tr_scheVal species=\"FAGR\">1.0</tr_scheVal>");
      oOut.write("</tr_stdCrownHtExp>");
      oOut.write("<tr_slopeOfHeight-Diam10>");
      oOut.write("<tr_sohdVal species=\"ACSA\">0.03418</tr_sohdVal>");
      oOut.write("<tr_sohdVal species=\"ACRU\">0.0269</tr_sohdVal>");
      oOut.write("<tr_sohdVal species=\"QURU\">0.02871</tr_sohdVal>");
      oOut.write("<tr_sohdVal species=\"PRSE\">0.02815</tr_sohdVal>");
      oOut.write("<tr_sohdVal species=\"PIST\">0.02871</tr_sohdVal>");
      oOut.write("<tr_sohdVal species=\"TSCA\">0.03224</tr_sohdVal>");
      oOut.write("<tr_sohdVal species=\"FRAM\">0.04796</tr_sohdVal>");
      oOut.write("<tr_sohdVal species=\"BEAL\">0.04681</tr_sohdVal>");
      oOut.write("<tr_sohdVal species=\"FAGR\">0.04101</tr_sohdVal>");
      oOut.write("</tr_slopeOfHeight-Diam10>");
      oOut.write("<tr_minAdultDBH>");
      oOut.write("<tr_madVal species=\"ACSA\">10.0</tr_madVal>");
      oOut.write("<tr_madVal species=\"ACRU\">10.0</tr_madVal>");
      oOut.write("<tr_madVal species=\"QURU\">10.0</tr_madVal>");
      oOut.write("<tr_madVal species=\"PRSE\">10.0</tr_madVal>");
      oOut.write("<tr_madVal species=\"PIST\">10.0</tr_madVal>");
      oOut.write("<tr_madVal species=\"TSCA\">10.0</tr_madVal>");
      oOut.write("<tr_madVal species=\"FRAM\">10.0</tr_madVal>");
      oOut.write("<tr_madVal species=\"BEAL\">10.0</tr_madVal>");
      oOut.write("<tr_madVal species=\"FAGR\">10.0</tr_madVal>");
      oOut.write("</tr_minAdultDBH>");
      oOut.write("<tr_slopeOfAsymHeight>");
      oOut.write("<tr_soahVal species=\"ACSA\">0.0299</tr_soahVal>");
      oOut.write("<tr_soahVal species=\"ACRU\">0.0241</tr_soahVal>");
      oOut.write("<tr_soahVal species=\"QURU\">0.0263</tr_soahVal>");
      oOut.write("<tr_soahVal species=\"PRSE\">0.0264</tr_soahVal>");
      oOut.write("<tr_soahVal species=\"PIST\">0.0264</tr_soahVal>");
      oOut.write("<tr_soahVal species=\"TSCA\">0.0333</tr_soahVal>");
      oOut.write("<tr_soahVal species=\"FRAM\">0.0352</tr_soahVal>");
      oOut.write("<tr_soahVal species=\"BEAL\">0.0347</tr_soahVal>");
      oOut.write("<tr_soahVal species=\"FAGR\">0.0454</tr_soahVal>");
      oOut.write("</tr_slopeOfAsymHeight>");

      oOut.write("<tr_treemap>");
      oOut.write("<tm_treeSettings sp=\"ACSA\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">3</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">2</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"float 1\">1</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("<tm_intCodes>");
      oOut.write("<tm_intCode label=\"int 1\">0</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 2\">1</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 3\">3</tm_intCode>");
      oOut.write("</tm_intCodes>");
      oOut.write("<tm_charCodes>");
      oOut.write("<tm_charCode label=\"char 1\">0</tm_charCode>");
      oOut.write("<tm_charCode label=\"char 2\">1</tm_charCode>");
      oOut.write("</tm_charCodes>");
      oOut.write("<tm_boolCodes>");
      oOut.write("<tm_boolCode label=\"bool 1\">0</tm_boolCode>");
      oOut.write("<tm_boolCode label=\"bool 2\">1</tm_boolCode>");
      oOut.write("</tm_boolCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"ACRU\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"float 1\">3</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("<tm_intCodes>");
      oOut.write("<tm_intCode label=\"int 1\">3</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 2\">2</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 3\">1</tm_intCode>");
      oOut.write("</tm_intCodes>");
      oOut.write("<tm_charCodes>");
      oOut.write("<tm_charCode label=\"char 1\">1</tm_charCode>");
      oOut.write("<tm_charCode label=\"char 2\">0</tm_charCode>");
      oOut.write("</tm_charCodes>");
      oOut.write("<tm_boolCodes>");
      oOut.write("<tm_boolCode label=\"bool 1\">1</tm_boolCode>");
      oOut.write("<tm_boolCode label=\"bool 2\">0</tm_boolCode>");
      oOut.write("</tm_boolCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"QURU\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"float 1\">3</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("<tm_intCodes>");
      oOut.write("<tm_intCode label=\"int 1\">3</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 2\">2</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 3\">1</tm_intCode>");
      oOut.write("</tm_intCodes>");
      oOut.write("<tm_charCodes>");
      oOut.write("<tm_charCode label=\"char 1\">1</tm_charCode>");
      oOut.write("<tm_charCode label=\"char 2\">0</tm_charCode>");
      oOut.write("</tm_charCodes>");
      oOut.write("<tm_boolCodes>");
      oOut.write("<tm_boolCode label=\"bool 1\">1</tm_boolCode>");
      oOut.write("<tm_boolCode label=\"bool 2\">0</tm_boolCode>");
      oOut.write("</tm_boolCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"ACRU\" tp=\"1\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Height\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("<tm_intCodes>");
      oOut.write("<tm_intCode label=\"int 1\">0</tm_intCode>");
      oOut.write("</tm_intCodes>");
      oOut.write("<tm_charCodes>");
      oOut.write("<tm_charCode label=\"char 1\">0</tm_charCode>");
      oOut.write("</tm_charCodes>");
      oOut.write("<tm_boolCodes>");
      oOut.write("<tm_boolCode label=\"bool 1\">0</tm_boolCode>");
      oOut.write("</tm_boolCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"QURU\" tp=\"1\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Height\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"QURU\" tp=\"3\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"ACRU\" tp=\"3\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"ACSA\" tp=\"3\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");

      //Trees
      //One of each type/species combo with each data member
      //Tree 1
      oOut.write("<tree sp=\"0\" tp=\"2\">");
      oOut.write("<fl c=\"3\">0.981445</fl>"); //X
      oOut.write("<fl c=\"2\">41.499001</fl>"); //Y
      oOut.write("<fl c=\"0\">89.027000</fl>"); //DBH
      oOut.write("<fl c=\"1\">1.1</fl>"); //float 1
      oOut.write("<int c=\"0\">1</int>"); //int 1
      oOut.write("<int c=\"1\">2</int>"); //int 2
      oOut.write("<int c=\"3\">3</int>"); //int 3
      oOut.write("<ch c=\"0\">char val 1</ch>"); //char 1
      oOut.write("<ch c=\"1\">char val 2</ch>"); //char 2
      oOut.write("<bl c=\"0\">true</bl>"); //bool 1
      oOut.write("<bl c=\"1\">false</bl>"); //bool 2
      oOut.write("</tree>");

      //Tree 2
      oOut.write("<tree sp=\"1\" tp=\"2\">");
      oOut.write("<fl c=\"0\">3.149410</fl>"); //X
      oOut.write("<fl c=\"1\">11.313500</fl>"); //Y
      oOut.write("<fl c=\"2\">89.988998</fl>"); //DBH
      oOut.write("<fl c=\"3\">2.1</fl>"); //float 1
      oOut.write("<int c=\"3\">4</int>"); //int 1
      oOut.write("<int c=\"2\">5</int>"); //int 2
      oOut.write("<int c=\"1\">6</int>"); //int 3
      oOut.write("<ch c=\"1\">char val 2.1</ch>"); //char 1
      oOut.write("<ch c=\"0\">char val 2.2</ch>"); //char 2
      oOut.write("<bl c=\"1\">true</bl>"); //bool 1
      oOut.write("<bl c=\"0\">false</bl>"); //bool 2
      oOut.write("</tree>");

      //Tree 3
      oOut.write("<tree sp=\"2\" tp=\"2\">");
      oOut.write("<fl c=\"0\">10.981445</fl>"); //X
      oOut.write("<fl c=\"1\">51.499001</fl>"); //Y
      oOut.write("<fl c=\"2\">9.027000</fl>"); //DBH
      oOut.write("<fl c=\"3\">3.1</fl>"); //float 1
      oOut.write("<int c=\"3\">7</int>"); //int 1
      oOut.write("<int c=\"2\">8</int>"); //int 2
      oOut.write("<int c=\"1\">9</int>"); //int 3
      oOut.write("<ch c=\"1\">char val 3.1</ch>"); //char 1
      oOut.write("<ch c=\"0\">char val 3.2</ch>"); //char 2
      oOut.write("<bl c=\"1\">false</bl>"); //bool 1
      oOut.write("<bl c=\"0\">true</bl>"); //bool 2
      oOut.write("</tree>");

      //Tree 4
      oOut.write("<tree sp=\"1\" tp=\"1\">");
      oOut.write("<fl c=\"0\">1.923830</fl>"); //X
      oOut.write("<fl c=\"1\">55.781200</fl>"); //Y
      oOut.write("<fl c=\"2\">43.942001</fl>"); //Height
      oOut.write("<int c=\"0\">10</int>"); //int 1
      oOut.write("<ch c=\"0\">char val 4.1</ch>"); //char 1
      oOut.write("<bl c=\"0\">false</bl>"); //bool 1
      oOut.write("</tree>");

      //Tree 5
      oOut.write("<tree sp=\"2\" tp=\"1\">");
      oOut.write("<fl c=\"0\">0.810547</fl>"); //X
      oOut.write("<fl c=\"1\">54.584999</fl>"); //Y
      oOut.write("<fl c=\"2\">81.760002</fl>"); //Height
      oOut.write("</tree>");

      //Tree 6
      oOut.write("<tree sp=\"2\" tp=\"3\">");
      oOut.write("<fl c=\"0\">7.470700</fl>"); //X
      oOut.write("<fl c=\"1\">93.603500</fl>"); //Y
      oOut.write("<fl c=\"2\">41.171001</fl>"); //DBH
      oOut.write("</tree>");

      //Tree 7
      oOut.write("<tree sp=\"1\" tp=\"3\">");
      oOut.write("<fl c=\"0\">6.528320</fl>"); //X
      oOut.write("<fl c=\"1\">102.182999</fl>"); //Y
      oOut.write("<fl c=\"2\">43.853001</fl>"); //DBH
      oOut.write("</tree>");

      //Tree 8
      oOut.write("<tree sp=\"0\" tp=\"3\">");
      oOut.write("<fl c=\"0\">0.732422</fl>"); //X
      oOut.write("<fl c=\"1\">99.418900</fl>"); //Y
      oOut.write("<fl c=\"2\">83.000999</fl>"); //DBH
      oOut.write("</tree>");

      oOut.write("</tr_treemap>");
      oOut.write("</trees>");
      oOut.write("</paramFile>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }

  /**
   * Writes a tree map file.  There are all four kinds of data members,
   * with different codes from those written in WriteXMLTreeMap1.  The
   * species list is also in a different order.  This
   * does not write a full parameter file, but a fake timestep file.
   * @return The file name.
   * @throws ModelException
   *
   * <br>Edit history:
   * <br>------------------
   * <br>April 28, 2004: Submitted in beta version (LEM)
   */
  protected String WriteXMLTreeMap2() throws ModelException {
    try {
      String sFileName = "\\testtreemap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      oOut.write("<tr_treemap>");

      //Species list
      oOut.write("<tm_speciesList>");
      oOut.write("<tm_species speciesName=\"QURU\"/>");
      oOut.write("<tm_species speciesName=\"ACRU\"/>");
      oOut.write("<tm_species speciesName=\"ACSA\"/>");
      oOut.write("</tm_speciesList>");

      oOut.write("<tm_treeSettings sp=\"ACSA\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"float 1\">3</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("<tm_intCodes>");
      oOut.write("<tm_intCode label=\"int 1\">2</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 2\">0</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 3\">1</tm_intCode>");
      oOut.write("</tm_intCodes>");
      oOut.write("<tm_charCodes>");
      oOut.write("<tm_charCode label=\"char 1\">1</tm_charCode>");
      oOut.write("<tm_charCode label=\"char 2\">0</tm_charCode>");
      oOut.write("</tm_charCodes>");
      oOut.write("<tm_boolCodes>");
      oOut.write("<tm_boolCode label=\"bool 1\">1</tm_boolCode>");
      oOut.write("<tm_boolCode label=\"bool 2\">0</tm_boolCode>");
      oOut.write("</tm_boolCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"ACRU\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">3</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">2</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"float 1\">0</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("<tm_intCodes>");
      oOut.write("<tm_intCode label=\"int 1\">0</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 2\">1</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 3\">2</tm_intCode>");
      oOut.write("</tm_intCodes>");
      oOut.write("<tm_charCodes>");
      oOut.write("<tm_charCode label=\"char 1\">1</tm_charCode>");
      oOut.write("<tm_charCode label=\"char 2\">0</tm_charCode>");
      oOut.write("</tm_charCodes>");
      oOut.write("<tm_boolCodes>");
      oOut.write("<tm_boolCode label=\"bool 1\">1</tm_boolCode>");
      oOut.write("<tm_boolCode label=\"bool 2\">0</tm_boolCode>");
      oOut.write("</tm_boolCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"QURU\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">2</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">3</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"float 1\">1</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("<tm_intCodes>");
      oOut.write("<tm_intCode label=\"int 1\">2</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 2\">1</tm_intCode>");
      oOut.write("<tm_intCode label=\"int 3\">3</tm_intCode>");
      oOut.write("</tm_intCodes>");
      oOut.write("<tm_charCodes>");
      oOut.write("<tm_charCode label=\"char 1\">0</tm_charCode>");
      oOut.write("<tm_charCode label=\"char 2\">2</tm_charCode>");
      oOut.write("</tm_charCodes>");
      oOut.write("<tm_boolCodes>");
      oOut.write("<tm_boolCode label=\"bool 1\">2</tm_boolCode>");
      oOut.write("<tm_boolCode label=\"bool 2\">3</tm_boolCode>");
      oOut.write("</tm_boolCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"ACRU\" tp=\"1\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">2</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Height\">0</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("<tm_intCodes>");
      oOut.write("<tm_intCode label=\"int 1\">1</tm_intCode>");
      oOut.write("</tm_intCodes>");
      oOut.write("<tm_charCodes>");
      oOut.write("<tm_charCode label=\"char 1\">1</tm_charCode>");
      oOut.write("</tm_charCodes>");
      oOut.write("<tm_boolCodes>");
      oOut.write("<tm_boolCode label=\"bool 1\">1</tm_boolCode>");
      oOut.write("</tm_boolCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"QURU\" tp=\"1\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Height\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"QURU\" tp=\"3\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"ACRU\" tp=\"3\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");
      oOut.write("<tm_treeSettings sp=\"ACSA\" tp=\"3\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");

      //Trees
      //One of each type/species combo with each data member
      //Tree 1
      oOut.write("<tree sp=\"2\" tp=\"2\">");
      oOut.write("<fl c=\"0\">0.981445</fl>"); //X
      oOut.write("<fl c=\"1\">41.499001</fl>"); //Y
      oOut.write("<fl c=\"2\">89.027000</fl>"); //DBH
      oOut.write("<fl c=\"3\">1.1</fl>"); //float 1
      oOut.write("<int c=\"2\">1</int>"); //int 1
      oOut.write("<int c=\"0\">2</int>"); //int 2
      oOut.write("<int c=\"1\">3</int>"); //int 3
      oOut.write("<ch c=\"1\">char val 1</ch>"); //char 1
      oOut.write("<ch c=\"0\">char val 2</ch>"); //char 2
      oOut.write("<bl c=\"1\">true</bl>"); //bool 1
      oOut.write("<bl c=\"0\">false</bl>"); //bool 2
      oOut.write("</tree>");

      //Tree 2
      oOut.write("<tree sp=\"1\" tp=\"2\">");
      oOut.write("<fl c=\"3\">3.149410</fl>"); //X
      oOut.write("<fl c=\"2\">11.313500</fl>"); //Y
      oOut.write("<fl c=\"1\">89.988998</fl>"); //DBH
      oOut.write("<fl c=\"0\">2.1</fl>"); //float 1
      oOut.write("<int c=\"0\">4</int>"); //int 1
      oOut.write("<int c=\"1\">5</int>"); //int 2
      oOut.write("<int c=\"2\">6</int>"); //int 3
      oOut.write("<ch c=\"1\">char val 2.1</ch>"); //char 1
      oOut.write("<ch c=\"0\">char val 2.2</ch>"); //char 2
      oOut.write("<bl c=\"1\">true</bl>"); //bool 1
      oOut.write("<bl c=\"0\">false</bl>"); //bool 2
      oOut.write("</tree>");

      //Tree 3
      oOut.write("<tree sp=\"0\" tp=\"2\">");
      oOut.write("<fl c=\"2\">10.981445</fl>"); //X
      oOut.write("<fl c=\"0\">51.499001</fl>"); //Y
      oOut.write("<fl c=\"3\">9.027000</fl>"); //DBH
      oOut.write("<fl c=\"1\">3.1</fl>"); //float 1
      oOut.write("<int c=\"2\">7</int>"); //int 1
      oOut.write("<int c=\"1\">8</int>"); //int 2
      oOut.write("<int c=\"3\">9</int>"); //int 3
      oOut.write("<ch c=\"0\">char val 3.1</ch>"); //char 1
      oOut.write("<ch c=\"2\">char val 3.2</ch>"); //char 2
      oOut.write("<bl c=\"2\">false</bl>"); //bool 1
      oOut.write("<bl c=\"3\">true</bl>"); //bool 2
      oOut.write("</tree>");

      //Tree 4
      oOut.write("<tree sp=\"1\" tp=\"1\">");
      oOut.write("<fl c=\"2\">1.923830</fl>"); //X
      oOut.write("<fl c=\"1\">55.781200</fl>"); //Y
      oOut.write("<fl c=\"0\">43.942001</fl>"); //Height
      oOut.write("<int c=\"1\">10</int>"); //int 1
      oOut.write("<ch c=\"1\">char val 4.1</ch>"); //char 1
      oOut.write("<bl c=\"1\">false</bl>"); //bool 1
      oOut.write("</tree>");

      //Tree 5
      oOut.write("<tree sp=\"0\" tp=\"1\">");
      oOut.write("<fl c=\"0\">0.810547</fl>"); //X
      oOut.write("<fl c=\"1\">54.584999</fl>"); //Y
      oOut.write("<fl c=\"2\">81.760002</fl>"); //Height
      oOut.write("</tree>");

      //Tree 6
      oOut.write("<tree sp=\"0\" tp=\"3\">");
      oOut.write("<fl c=\"0\">7.470700</fl>"); //X
      oOut.write("<fl c=\"1\">93.603500</fl>"); //Y
      oOut.write("<fl c=\"2\">41.171001</fl>"); //DBH
      oOut.write("</tree>");

      //Tree 7
      oOut.write("<tree sp=\"1\" tp=\"3\">");
      oOut.write("<fl c=\"0\">6.528320</fl>"); //X
      oOut.write("<fl c=\"1\">102.182999</fl>"); //Y
      oOut.write("<fl c=\"2\">43.853001</fl>"); //DBH
      oOut.write("</tree>");

      //Tree 8
      oOut.write("<tree sp=\"2\" tp=\"3\">");
      oOut.write("<fl c=\"0\">0.732422</fl>"); //X
      oOut.write("<fl c=\"1\">99.418900</fl>"); //Y
      oOut.write("<fl c=\"2\">83.000999</fl>"); //DBH
      oOut.write("</tree>");

      oOut.write("</tr_treemap>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }

  /**
   * Writes a bad Tree map file. One of the X values is beyond the plot.
   * @return The file name.
   * @throws ModelException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>July 26, 2004: Created (LEM)
   */
  protected String WriteErrorXMLTreeMap1() throws ModelException {
    try {
      String sFileName = "\\testtreemap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      oOut.write("<tr_treemap>");

      //Species list
      oOut.write("<tm_speciesList>");
      oOut.write("<tm_species speciesName=\"QURU\"/>");
      oOut.write("<tm_species speciesName=\"ACRU\"/>");
      oOut.write("<tm_species speciesName=\"ACSA\"/>");
      oOut.write("</tm_speciesList>");

      oOut.write("<tm_treeSettings sp=\"ACSA\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">3</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");

      //Trees
      //One of each type/species combo with each data member
      //Tree 1
      oOut.write("<tree sp=\"2\" tp=\"2\">");
      oOut.write("<fl c=\"3\">200.981445</fl>"); //X
      oOut.write("<fl c=\"1\">41.499001</fl>"); //Y
      oOut.write("<fl c=\"2\">89.027000</fl>"); //DBH
      oOut.write("</tree>");

      oOut.write("</tr_treemap>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }

  /**
   * Writes a bad tree map file. One of the X values is negative the plot.
   * @return The file name.
   * @throws ModelException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>July 26, 2004: Created (LEM)
   */
  protected String WriteErrorXMLTreeMap2() throws ModelException {
    try {
      String sFileName = "\\testtreemap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      oOut.write("<tr_treemap>");

      //Species list
      oOut.write("<tm_speciesList>");
      oOut.write("<tm_species speciesName=\"QURU\"/>");
      oOut.write("<tm_species speciesName=\"ACRU\"/>");
      oOut.write("<tm_species speciesName=\"ACSA\"/>");
      oOut.write("</tm_speciesList>");

      oOut.write("<tm_treeSettings sp=\"ACSA\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">3</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">1</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");

      //Trees
      //One of each type/species combo with each data member
      //Tree 1
      oOut.write("<tree sp=\"2\" tp=\"2\">");
      oOut.write("<fl c=\"3\">-0.981445</fl>"); //X
      oOut.write("<fl c=\"1\">41.499001</fl>"); //Y
      oOut.write("<fl c=\"2\">89.027000</fl>"); //DBH
      oOut.write("</tree>");

      oOut.write("</tr_treemap>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }

  /**
   * Writes a bad Tree map file. One of the Y values is beyond the plot.
   * @return The file name.
   * @throws ModelException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>July 26, 2004: Created (LEM)
   */
  protected String WriteErrorXMLTreeMap3() throws ModelException {
    try {
      String sFileName = "\\testtreemap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      oOut.write("<tr_treemap>");

      //Species list
      oOut.write("<tm_speciesList>");
      oOut.write("<tm_species speciesName=\"QURU\"/>");
      oOut.write("<tm_species speciesName=\"ACRU\"/>");
      oOut.write("<tm_species speciesName=\"ACSA\"/>");
      oOut.write("</tm_speciesList>");

      oOut.write("<tm_treeSettings sp=\"ACSA\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">3</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");

      //Trees
      //One of each type/species combo with each data member
      //Tree 1
      oOut.write("<tree sp=\"2\" tp=\"2\">");
      oOut.write("<fl c=\"0\">0.981445</fl>"); //X
      oOut.write("<fl c=\"3\">241.499001</fl>"); //Y
      oOut.write("<fl c=\"2\">89.027000</fl>"); //DBH
      oOut.write("</tree>");

      oOut.write("</tr_treemap>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }

  /**
   * Writes a bad tree map file. One of the Y values is beyond the plot.
   * @return The file name.
   * @throws ModelException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>July 26, 2004: Created (LEM)
   */
  protected String WriteErrorXMLTreeMap4() throws ModelException {
    try {
      String sFileName = "\\testtreemap2.xml";
      FileWriter oOut = new FileWriter(sFileName);

      oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
      oOut.write("<timestepRundata fileCode=\"06010701\">");
      oOut.write("<rt_timestep>0</rt_timestep>");

      oOut.write("<tr_treemap>");

      //Species list
      oOut.write("<tm_speciesList>");
      oOut.write("<tm_species speciesName=\"QURU\"/>");
      oOut.write("<tm_species speciesName=\"ACRU\"/>");
      oOut.write("<tm_species speciesName=\"ACSA\"/>");
      oOut.write("</tm_speciesList>");

      oOut.write("<tm_treeSettings sp=\"ACSA\" tp=\"2\">");
      oOut.write("<tm_floatCodes>");
      oOut.write("<tm_floatCode label=\"X\">0</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"Y\">3</tm_floatCode>");
      oOut.write("<tm_floatCode label=\"DBH\">2</tm_floatCode>");
      oOut.write("</tm_floatCodes>");
      oOut.write("</tm_treeSettings>");

      //Trees
      //One of each type/species combo with each data member
      //Tree 1
      oOut.write("<tree sp=\"2\" tp=\"2\">");
      oOut.write("<fl c=\"0\">0.981445</fl>"); //X
      oOut.write("<fl c=\"3\">-41.499001</fl>"); //Y
      oOut.write("<fl c=\"2\">89.027000</fl>"); //DBH
      oOut.write("</tree>");

      oOut.write("</tr_treemap>");
      oOut.write("</timestepRundata>");

      oOut.close();
      return sFileName;
    }
    catch (java.io.IOException oE) {
      throw(new ModelException(ErrorGUI.BAD_FILE, "", oE.getMessage()));
    }
  }

  /**
   * Writes a parameter file for validating tree map reading.
   * @return String Filename written.
   * @throws IOException if there is a problem writing to the file.
   */
  private String WriteTreeMapParFile() throws java.io.IOException {

    String sFileName = "\\loratestxml1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>2</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>100.0</plot_lenX>");
    oOut.write("<plot_lenY>150.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Red_Maple\" />");
    oOut.write("<tr_species speciesName=\"Oak\" />");
    oOut.write("<tr_species speciesName=\"Beech\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Red_Maple\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Oak\">39.54</tr_chVal>");
    oOut.write("<tr_chVal species=\"Beech\">45</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Red_Maple\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Oak\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Beech\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Red_Maple\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Oak\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Beech\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Red_Maple\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Oak\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Beech\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Red_Maple\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Oak\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Beech\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Red_Maple\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Oak\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Beech\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Red_Maple\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Oak\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Beech\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Red_Maple\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Oak\">0.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Beech\">0.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Red_Maple\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Oak\">0.0241</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Beech\">0.0263</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</trees>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Oak\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Oak\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"Beech\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Beech\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

}
